# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_REGISTRATION_SY_NQUICK_SH_METADATA = Metadata(
    id="9bf492799cfd56f1976ba1ef5ef603c832961ac8.boutiques",
    name="antsRegistrationSyNQuick.sh",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsRegistrationSyNquickShParamsDictNoTag = typing.TypedDict('_AntsRegistrationSyNquickShParamsDictNoTag', {
    "dimensionality": typing.Literal[2, 3],
    "fixed_image": InputPathType,
    "moving_image": InputPathType,
    "output_prefix": str,
    "transform_type": typing.NotRequired[typing.Literal["s", "b"] | None],
})
AntsRegistrationSyNquickShParamsDictTagged = typing.TypedDict('AntsRegistrationSyNquickShParamsDictTagged', {
    "@type": typing.Literal["ants/antsRegistrationSyNQuick.sh"],
    "dimensionality": typing.Literal[2, 3],
    "fixed_image": InputPathType,
    "moving_image": InputPathType,
    "output_prefix": str,
    "transform_type": typing.NotRequired[typing.Literal["s", "b"] | None],
})
AntsRegistrationSyNquickShParamsDict = _AntsRegistrationSyNquickShParamsDictNoTag | AntsRegistrationSyNquickShParamsDictTagged


class AntsRegistrationSyNquickShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsRegistrationSyNquickShParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_transform: OutputPathType
    """Affine transformation matrix resulting from registration."""
    output_warp: OutputPathType
    """Warp field resulting from the registration."""
    output_inverse_warp: OutputPathType
    """Inverse warp field resulting from the registration."""
    output_warped_image: OutputPathType
    """Warped moving image."""


def ants_registration_sy_nquick_sh_params(
    dimensionality: typing.Literal[2, 3],
    fixed_image: InputPathType,
    moving_image: InputPathType,
    output_prefix: str,
    transform_type: typing.Literal["s", "b"] | None = None,
) -> AntsRegistrationSyNquickShParamsDictTagged:
    """
    Build parameters.
    
    Args:
        dimensionality: Dimensionality of the images (2 or 3).
        fixed_image: Fixed image to which the moving image is registered.
        moving_image: Moving image that is registered to the fixed image.
        output_prefix: Prefix for the output files.
        transform_type: Type of transform: 's' for SyN, 'b' for B-spline SyN.\
            Default is 's'.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsRegistrationSyNQuick.sh",
        "dimensionality": dimensionality,
        "fixed_image": fixed_image,
        "moving_image": moving_image,
        "output_prefix": output_prefix,
    }
    if transform_type is not None:
        params["transform_type"] = transform_type
    return params


def ants_registration_sy_nquick_sh_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsRegistrationSyNquickShParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("dimensionality", None) is None:
        raise StyxValidationError("`dimensionality` must not be None")
    if not isinstance(params["dimensionality"], int):
        raise StyxValidationError(f'`dimensionality` has the wrong type: Received `{type(params.get("dimensionality", None))}` expected `typing.Literal[2, 3]`')
    if params["dimensionality"] not in [2, 3]:
        raise StyxValidationError("Parameter `dimensionality` must be one of [2, 3]")
    if params.get("fixed_image", None) is None:
        raise StyxValidationError("`fixed_image` must not be None")
    if not isinstance(params["fixed_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`fixed_image` has the wrong type: Received `{type(params.get("fixed_image", None))}` expected `InputPathType`')
    if params.get("moving_image", None) is None:
        raise StyxValidationError("`moving_image` must not be None")
    if not isinstance(params["moving_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`moving_image` has the wrong type: Received `{type(params.get("moving_image", None))}` expected `InputPathType`')
    if params.get("output_prefix", None) is None:
        raise StyxValidationError("`output_prefix` must not be None")
    if not isinstance(params["output_prefix"], str):
        raise StyxValidationError(f'`output_prefix` has the wrong type: Received `{type(params.get("output_prefix", None))}` expected `str`')
    if params.get("transform_type", None) is not None:
        if not isinstance(params["transform_type"], str):
            raise StyxValidationError(f'`transform_type` has the wrong type: Received `{type(params.get("transform_type", None))}` expected `typing.Literal["s", "b"] | None`')
        if params["transform_type"] not in ["s", "b"]:
            raise StyxValidationError("Parameter `transform_type` must be one of [\"s\", \"b\"]")


def ants_registration_sy_nquick_sh_cargs(
    params: AntsRegistrationSyNquickShParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsRegistrationSyNQuick.sh")
    cargs.extend([
        "-d",
        str(params.get("dimensionality", None))
    ])
    cargs.extend([
        "-f",
        execution.input_file(params.get("fixed_image", None))
    ])
    cargs.extend([
        "-m",
        execution.input_file(params.get("moving_image", None))
    ])
    cargs.extend([
        "-o",
        params.get("output_prefix", None)
    ])
    if params.get("transform_type", None) is not None:
        cargs.append(params.get("transform_type", None))
    return cargs


def ants_registration_sy_nquick_sh_outputs(
    params: AntsRegistrationSyNquickShParamsDict,
    execution: Execution,
) -> AntsRegistrationSyNquickShOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsRegistrationSyNquickShOutputs(
        root=execution.output_file("."),
        output_transform=execution.output_file(params.get("output_prefix", None) + "0GenericAffine.mat"),
        output_warp=execution.output_file(params.get("output_prefix", None) + "1Warp.nii.gz"),
        output_inverse_warp=execution.output_file(params.get("output_prefix", None) + "1InverseWarp.nii.gz"),
        output_warped_image=execution.output_file(params.get("output_prefix", None) + "Warped.nii.gz"),
    )
    return ret


def ants_registration_sy_nquick_sh_execute(
    params: AntsRegistrationSyNquickShParamsDict,
    runner: Runner | None = None,
) -> AntsRegistrationSyNquickShOutputs:
    """
    antsRegistrationSyNQuick.sh
    
    A script to quickly compute a SyN-based registration between two images
    using ANTS.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsRegistrationSyNquickShOutputs`).
    """
    ants_registration_sy_nquick_sh_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_REGISTRATION_SY_NQUICK_SH_METADATA)
    params = execution.params(params)
    cargs = ants_registration_sy_nquick_sh_cargs(params, execution)
    ret = ants_registration_sy_nquick_sh_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_registration_sy_nquick_sh(
    dimensionality: typing.Literal[2, 3],
    fixed_image: InputPathType,
    moving_image: InputPathType,
    output_prefix: str,
    transform_type: typing.Literal["s", "b"] | None = None,
    runner: Runner | None = None,
) -> AntsRegistrationSyNquickShOutputs:
    """
    antsRegistrationSyNQuick.sh
    
    A script to quickly compute a SyN-based registration between two images
    using ANTS.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        dimensionality: Dimensionality of the images (2 or 3).
        fixed_image: Fixed image to which the moving image is registered.
        moving_image: Moving image that is registered to the fixed image.
        output_prefix: Prefix for the output files.
        transform_type: Type of transform: 's' for SyN, 'b' for B-spline SyN.\
            Default is 's'.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsRegistrationSyNquickShOutputs`).
    """
    params = ants_registration_sy_nquick_sh_params(
        dimensionality=dimensionality,
        fixed_image=fixed_image,
        moving_image=moving_image,
        output_prefix=output_prefix,
        transform_type=transform_type,
    )
    return ants_registration_sy_nquick_sh_execute(params, runner)


__all__ = [
    "ANTS_REGISTRATION_SY_NQUICK_SH_METADATA",
    "AntsRegistrationSyNquickShOutputs",
    "AntsRegistrationSyNquickShParamsDict",
    "AntsRegistrationSyNquickShParamsDictTagged",
    "ants_registration_sy_nquick_sh",
    "ants_registration_sy_nquick_sh_execute",
    "ants_registration_sy_nquick_sh_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_JOINT_FUSION_METADATA = Metadata(
    id="b57f087fd4ad3a9b8192927bfb3d0e52df02a072.boutiques",
    name="antsJointFusion",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsJointFusionParamsDictNoTag = typing.TypedDict('_AntsJointFusionParamsDictNoTag', {
    "image_dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "target_image": list[InputPathType],
    "atlas_image": list[InputPathType],
    "atlas_segmentation": InputPathType,
    "alpha": typing.NotRequired[float | None],
    "beta": typing.NotRequired[float | None],
    "constrain_nonnegative": typing.NotRequired[bool | None],
    "patch_radius": typing.NotRequired[str | None],
    "patch_metric": typing.NotRequired[typing.Literal["PC", "MSQ"] | None],
    "search_radius": typing.NotRequired[str | None],
    "exclusion_image": typing.NotRequired[InputPathType | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "output": str,
    "verbose": typing.NotRequired[bool | None],
})
AntsJointFusionParamsDictTagged = typing.TypedDict('AntsJointFusionParamsDictTagged', {
    "@type": typing.Literal["ants/antsJointFusion"],
    "image_dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "target_image": list[InputPathType],
    "atlas_image": list[InputPathType],
    "atlas_segmentation": InputPathType,
    "alpha": typing.NotRequired[float | None],
    "beta": typing.NotRequired[float | None],
    "constrain_nonnegative": typing.NotRequired[bool | None],
    "patch_radius": typing.NotRequired[str | None],
    "patch_metric": typing.NotRequired[typing.Literal["PC", "MSQ"] | None],
    "search_radius": typing.NotRequired[str | None],
    "exclusion_image": typing.NotRequired[InputPathType | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "output": str,
    "verbose": typing.NotRequired[bool | None],
})
AntsJointFusionParamsDict = _AntsJointFusionParamsDictNoTag | AntsJointFusionParamsDictTagged


class AntsJointFusionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsJointFusionParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    label_fusion_image: OutputPathType
    """The output label fusion image."""
    intensity_fusion_image: OutputPathType
    """The output intensity fusion image format."""
    label_posterior_probability_image: OutputPathType
    """The output label posterior probability image format."""
    atlas_voting_weight_image: OutputPathType
    """The output atlas voting weight image format."""


def ants_joint_fusion_params(
    target_image: list[InputPathType],
    atlas_image: list[InputPathType],
    atlas_segmentation: InputPathType,
    output: str,
    image_dimensionality: typing.Literal[2, 3, 4] | None = None,
    alpha: float | None = None,
    beta: float | None = None,
    constrain_nonnegative: bool | None = None,
    patch_radius: str | None = None,
    patch_metric: typing.Literal["PC", "MSQ"] | None = None,
    search_radius: str | None = None,
    exclusion_image: InputPathType | None = None,
    mask_image: InputPathType | None = None,
    verbose: bool | None = None,
) -> AntsJointFusionParamsDictTagged:
    """
    Build parameters.
    
    Args:
        target_image: The target image (or multimodal target images) assumed to\
            be aligned to a common image domain.
        atlas_image: The atlas image (or multimodal atlas images) assumed to be\
            aligned to a common image domain.
        atlas_segmentation: The atlas segmentation images. For performing label\
            fusion the number of specified segmentations should be identical to the\
            number of atlas image sets.
        output: The output is the intensity and/or label fusion image.\
            Additional optional outputs include the label posterior probability\
            images and the atlas voting weight images.
        image_dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, the program tries to\
            infer the dimensionality from the input image.
        alpha: Regularization term added to matrix Mx for calculating the\
            inverse. Default = 0.1.
        beta: Exponent for mapping intensity difference to the joint error.\
            Default = 2.0.
        constrain_nonnegative: Constrain solution to non-negative weights.
        patch_radius: Patch radius for similarity measures. Default = 2x2x2.
        patch_metric: Metric to be used in determining the most similar\
            neighborhood patch. Options include Pearson's correlation (PC) and mean\
            squares (MSQ). Default = PC (Pearson correlation).
        search_radius: Search radius for similarity measures. Default = 3x3x3.\
            One can also specify an image where the value at the voxel specifies\
            the isotropic search radius at that voxel.
        exclusion_image: Specify an exclusion region for the given label.
        mask_image: If a mask image is specified, fusion is only performed in\
            the mask region.
        verbose: Verbose output.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsJointFusion",
        "target_image": target_image,
        "atlas_image": atlas_image,
        "atlas_segmentation": atlas_segmentation,
        "output": output,
    }
    if image_dimensionality is not None:
        params["image_dimensionality"] = image_dimensionality
    if alpha is not None:
        params["alpha"] = alpha
    if beta is not None:
        params["beta"] = beta
    if constrain_nonnegative is not None:
        params["constrain_nonnegative"] = constrain_nonnegative
    if patch_radius is not None:
        params["patch_radius"] = patch_radius
    if patch_metric is not None:
        params["patch_metric"] = patch_metric
    if search_radius is not None:
        params["search_radius"] = search_radius
    if exclusion_image is not None:
        params["exclusion_image"] = exclusion_image
    if mask_image is not None:
        params["mask_image"] = mask_image
    if verbose is not None:
        params["verbose"] = verbose
    return params


def ants_joint_fusion_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsJointFusionParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimensionality", None) is not None:
        if not isinstance(params["image_dimensionality"], int):
            raise StyxValidationError(f'`image_dimensionality` has the wrong type: Received `{type(params.get("image_dimensionality", None))}` expected `typing.Literal[2, 3, 4] | None`')
        if params["image_dimensionality"] not in [2, 3, 4]:
            raise StyxValidationError("Parameter `image_dimensionality` must be one of [2, 3, 4]")
    if params.get("target_image", None) is None:
        raise StyxValidationError("`target_image` must not be None")
    if not isinstance(params["target_image"], list):
        raise StyxValidationError(f'`target_image` has the wrong type: Received `{type(params.get("target_image", None))}` expected `list[InputPathType]`')
    for e in params["target_image"]:
        if not isinstance(e, (pathlib.Path, str)):
            raise StyxValidationError(f'`target_image` has the wrong type: Received `{type(params.get("target_image", None))}` expected `list[InputPathType]`')
    if params.get("atlas_image", None) is None:
        raise StyxValidationError("`atlas_image` must not be None")
    if not isinstance(params["atlas_image"], list):
        raise StyxValidationError(f'`atlas_image` has the wrong type: Received `{type(params.get("atlas_image", None))}` expected `list[InputPathType]`')
    for e in params["atlas_image"]:
        if not isinstance(e, (pathlib.Path, str)):
            raise StyxValidationError(f'`atlas_image` has the wrong type: Received `{type(params.get("atlas_image", None))}` expected `list[InputPathType]`')
    if params.get("atlas_segmentation", None) is None:
        raise StyxValidationError("`atlas_segmentation` must not be None")
    if not isinstance(params["atlas_segmentation"], (pathlib.Path, str)):
        raise StyxValidationError(f'`atlas_segmentation` has the wrong type: Received `{type(params.get("atlas_segmentation", None))}` expected `InputPathType`')
    if params.get("alpha", None) is not None:
        if not isinstance(params["alpha"], (float, int)):
            raise StyxValidationError(f'`alpha` has the wrong type: Received `{type(params.get("alpha", None))}` expected `float | None`')
    if params.get("beta", None) is not None:
        if not isinstance(params["beta"], (float, int)):
            raise StyxValidationError(f'`beta` has the wrong type: Received `{type(params.get("beta", None))}` expected `float | None`')
    if params.get("constrain_nonnegative", None) is not None:
        if not isinstance(params["constrain_nonnegative"], bool):
            raise StyxValidationError(f'`constrain_nonnegative` has the wrong type: Received `{type(params.get("constrain_nonnegative", None))}` expected `bool | None`')
    if params.get("patch_radius", None) is not None:
        if not isinstance(params["patch_radius"], str):
            raise StyxValidationError(f'`patch_radius` has the wrong type: Received `{type(params.get("patch_radius", None))}` expected `str | None`')
    if params.get("patch_metric", None) is not None:
        if not isinstance(params["patch_metric"], str):
            raise StyxValidationError(f'`patch_metric` has the wrong type: Received `{type(params.get("patch_metric", None))}` expected `typing.Literal["PC", "MSQ"] | None`')
        if params["patch_metric"] not in ["PC", "MSQ"]:
            raise StyxValidationError("Parameter `patch_metric` must be one of [\"PC\", \"MSQ\"]")
    if params.get("search_radius", None) is not None:
        if not isinstance(params["search_radius"], str):
            raise StyxValidationError(f'`search_radius` has the wrong type: Received `{type(params.get("search_radius", None))}` expected `str | None`')
    if params.get("exclusion_image", None) is not None:
        if not isinstance(params["exclusion_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`exclusion_image` has the wrong type: Received `{type(params.get("exclusion_image", None))}` expected `InputPathType | None`')
    if params.get("mask_image", None) is not None:
        if not isinstance(params["mask_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask_image` has the wrong type: Received `{type(params.get("mask_image", None))}` expected `InputPathType | None`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')
    if params.get("verbose", None) is not None:
        if not isinstance(params["verbose"], bool):
            raise StyxValidationError(f'`verbose` has the wrong type: Received `{type(params.get("verbose", None))}` expected `bool | None`')


def ants_joint_fusion_cargs(
    params: AntsJointFusionParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsJointFusion")
    if params.get("image_dimensionality", None) is not None:
        cargs.extend([
            "--image-dimensionality",
            str(params.get("image_dimensionality", None))
        ])
    cargs.extend([
        "--target-image",
        *[execution.input_file(f) for f in params.get("target_image", None)]
    ])
    cargs.extend([
        "--atlas-image",
        *[execution.input_file(f) for f in params.get("atlas_image", None)]
    ])
    cargs.extend([
        "--atlas-segmentation",
        execution.input_file(params.get("atlas_segmentation", None))
    ])
    if params.get("alpha", None) is not None:
        cargs.extend([
            "--alpha",
            str(params.get("alpha", None))
        ])
    if params.get("beta", None) is not None:
        cargs.extend([
            "--beta",
            str(params.get("beta", None))
        ])
    if params.get("constrain_nonnegative", None) is not None:
        cargs.extend([
            "--constrain-nonnegative",
            ("1" if params.get("constrain_nonnegative", None) else "0")
        ])
    if params.get("patch_radius", None) is not None:
        cargs.extend([
            "--patch-radius",
            params.get("patch_radius", None)
        ])
    if params.get("patch_metric", None) is not None:
        cargs.extend([
            "--patch-metric",
            params.get("patch_metric", None)
        ])
    if params.get("search_radius", None) is not None:
        cargs.extend([
            "--search-radius",
            params.get("search_radius", None)
        ])
    if params.get("exclusion_image", None) is not None:
        cargs.extend([
            "--exclusion-image",
            execution.input_file(params.get("exclusion_image", None))
        ])
    if params.get("mask_image", None) is not None:
        cargs.extend([
            "--mask-image",
            execution.input_file(params.get("mask_image", None))
        ])
    cargs.extend([
        "--output",
        params.get("output", None)
    ])
    if params.get("verbose", None) is not None:
        cargs.extend([
            "--verbose",
            ("1" if params.get("verbose", None) else "0")
        ])
    return cargs


def ants_joint_fusion_outputs(
    params: AntsJointFusionParamsDict,
    execution: Execution,
) -> AntsJointFusionOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsJointFusionOutputs(
        root=execution.output_file("."),
        label_fusion_image=execution.output_file("[LABELFUSIONIMAGE]"),
        intensity_fusion_image=execution.output_file("[INTENSITYFUSIONIMAGEFILENAMEFORMAT]"),
        label_posterior_probability_image=execution.output_file("[LABELPOSTERIORPROBABILITYIMAGEFILENAMEFORMAT]"),
        atlas_voting_weight_image=execution.output_file("[ATLASVOTINGWEIGHTIMAGEFILENAMEFORMAT]"),
    )
    return ret


def ants_joint_fusion_execute(
    params: AntsJointFusionParamsDict,
    runner: Runner | None = None,
) -> AntsJointFusionOutputs:
    """
    antsJointFusion
    
    antsJointFusion is an image fusion algorithm developed by Hongzhi Wang and
    Paul Yushkevich. This implementation is based on Paul's original ITK-style
    implementation and Brian's ANTsR implementation. The original label fusion
    framework was extended to accommodate intensities.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsJointFusionOutputs`).
    """
    ants_joint_fusion_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_JOINT_FUSION_METADATA)
    params = execution.params(params)
    cargs = ants_joint_fusion_cargs(params, execution)
    ret = ants_joint_fusion_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_joint_fusion(
    target_image: list[InputPathType],
    atlas_image: list[InputPathType],
    atlas_segmentation: InputPathType,
    output: str,
    image_dimensionality: typing.Literal[2, 3, 4] | None = None,
    alpha: float | None = None,
    beta: float | None = None,
    constrain_nonnegative: bool | None = None,
    patch_radius: str | None = None,
    patch_metric: typing.Literal["PC", "MSQ"] | None = None,
    search_radius: str | None = None,
    exclusion_image: InputPathType | None = None,
    mask_image: InputPathType | None = None,
    verbose: bool | None = None,
    runner: Runner | None = None,
) -> AntsJointFusionOutputs:
    """
    antsJointFusion
    
    antsJointFusion is an image fusion algorithm developed by Hongzhi Wang and
    Paul Yushkevich. This implementation is based on Paul's original ITK-style
    implementation and Brian's ANTsR implementation. The original label fusion
    framework was extended to accommodate intensities.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        target_image: The target image (or multimodal target images) assumed to\
            be aligned to a common image domain.
        atlas_image: The atlas image (or multimodal atlas images) assumed to be\
            aligned to a common image domain.
        atlas_segmentation: The atlas segmentation images. For performing label\
            fusion the number of specified segmentations should be identical to the\
            number of atlas image sets.
        output: The output is the intensity and/or label fusion image.\
            Additional optional outputs include the label posterior probability\
            images and the atlas voting weight images.
        image_dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, the program tries to\
            infer the dimensionality from the input image.
        alpha: Regularization term added to matrix Mx for calculating the\
            inverse. Default = 0.1.
        beta: Exponent for mapping intensity difference to the joint error.\
            Default = 2.0.
        constrain_nonnegative: Constrain solution to non-negative weights.
        patch_radius: Patch radius for similarity measures. Default = 2x2x2.
        patch_metric: Metric to be used in determining the most similar\
            neighborhood patch. Options include Pearson's correlation (PC) and mean\
            squares (MSQ). Default = PC (Pearson correlation).
        search_radius: Search radius for similarity measures. Default = 3x3x3.\
            One can also specify an image where the value at the voxel specifies\
            the isotropic search radius at that voxel.
        exclusion_image: Specify an exclusion region for the given label.
        mask_image: If a mask image is specified, fusion is only performed in\
            the mask region.
        verbose: Verbose output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsJointFusionOutputs`).
    """
    params = ants_joint_fusion_params(
        image_dimensionality=image_dimensionality,
        target_image=target_image,
        atlas_image=atlas_image,
        atlas_segmentation=atlas_segmentation,
        alpha=alpha,
        beta=beta,
        constrain_nonnegative=constrain_nonnegative,
        patch_radius=patch_radius,
        patch_metric=patch_metric,
        search_radius=search_radius,
        exclusion_image=exclusion_image,
        mask_image=mask_image,
        output=output,
        verbose=verbose,
    )
    return ants_joint_fusion_execute(params, runner)


__all__ = [
    "ANTS_JOINT_FUSION_METADATA",
    "AntsJointFusionOutputs",
    "AntsJointFusionParamsDict",
    "AntsJointFusionParamsDictTagged",
    "ants_joint_fusion",
    "ants_joint_fusion_execute",
    "ants_joint_fusion_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_REGISTRATION_SY_N_SH_METADATA = Metadata(
    id="b8b4732301023521da1f1ecab58942846288f659.boutiques",
    name="antsRegistrationSyN.sh",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsRegistrationSyNShParamsDictNoTag = typing.TypedDict('_AntsRegistrationSyNShParamsDictNoTag', {
    "image_dimension": typing.Literal[2, 3],
    "fixed_image": InputPathType,
    "moving_image": InputPathType,
    "output_prefix": str,
    "threads": typing.NotRequired[int | None],
    "initial_transform": typing.NotRequired[list[str] | None],
    "transform_type": typing.NotRequired[typing.Literal["t", "r", "a", "s", "sr", "so", "b", "br", "bo"] | None],
    "radius": typing.NotRequired[int | None],
    "spline_distance": typing.NotRequired[int | None],
    "gradient_step": typing.NotRequired[float | None],
    "masks": typing.NotRequired[str | None],
    "precision_type": typing.NotRequired[typing.Literal["f", "d"] | None],
    "use_histogram_matching": typing.NotRequired[bool | None],
    "use_repro_mode": typing.NotRequired[bool | None],
    "collapse_output_transforms": typing.NotRequired[bool | None],
    "random_seed": typing.NotRequired[int | None],
})
AntsRegistrationSyNShParamsDictTagged = typing.TypedDict('AntsRegistrationSyNShParamsDictTagged', {
    "@type": typing.Literal["ants/antsRegistrationSyN.sh"],
    "image_dimension": typing.Literal[2, 3],
    "fixed_image": InputPathType,
    "moving_image": InputPathType,
    "output_prefix": str,
    "threads": typing.NotRequired[int | None],
    "initial_transform": typing.NotRequired[list[str] | None],
    "transform_type": typing.NotRequired[typing.Literal["t", "r", "a", "s", "sr", "so", "b", "br", "bo"] | None],
    "radius": typing.NotRequired[int | None],
    "spline_distance": typing.NotRequired[int | None],
    "gradient_step": typing.NotRequired[float | None],
    "masks": typing.NotRequired[str | None],
    "precision_type": typing.NotRequired[typing.Literal["f", "d"] | None],
    "use_histogram_matching": typing.NotRequired[bool | None],
    "use_repro_mode": typing.NotRequired[bool | None],
    "collapse_output_transforms": typing.NotRequired[bool | None],
    "random_seed": typing.NotRequired[int | None],
})
AntsRegistrationSyNShParamsDict = _AntsRegistrationSyNShParamsDictNoTag | AntsRegistrationSyNShParamsDictTagged


class AntsRegistrationSyNShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsRegistrationSyNShParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    affine_transform: OutputPathType
    """Affine transformation matrix for registration"""
    inverse_warp: OutputPathType
    """Inverse warp field for registration"""
    forward_warp: OutputPathType
    """Forward warp field for registration"""


def ants_registration_sy_n_sh_params(
    image_dimension: typing.Literal[2, 3],
    fixed_image: InputPathType,
    moving_image: InputPathType,
    output_prefix: str,
    threads: int | None = None,
    initial_transform: list[str] | None = None,
    transform_type: typing.Literal["t", "r", "a", "s", "sr", "so", "b", "br", "bo"] | None = None,
    radius: int | None = None,
    spline_distance: int | None = None,
    gradient_step: float | None = None,
    masks: str | None = None,
    precision_type: typing.Literal["f", "d"] | None = None,
    use_histogram_matching: bool | None = None,
    use_repro_mode: bool | None = None,
    collapse_output_transforms: bool | None = None,
    random_seed: int | None = None,
) -> AntsRegistrationSyNShParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: Image dimension: 2 or 3 (for 2 or 3-dimensional\
            registration of a single volume).
        fixed_image: Fixed image(s) or source image(s) or reference image(s).
        moving_image: Moving image(s) or target image(s).
        output_prefix: A prefix that is prepended to all output files.
        threads: Number of threads (default =\
            ITK_GLOBAL_DEFAULT_NUMBER_OF_THREADS if defined, otherwise 1).
        initial_transform: Initial transform(s) --- order specified on the\
            command line matters.
        transform_type: Transform type (default = 's'). Options:\
            - t: translation (1 stage)\
            - r: rigid (1 stage)\
            - a: rigid + affine (2 stages)\
            - s: rigid + affine + deformable syn (3 stages)\
            - sr: rigid + deformable syn (2 stages)\
            - so: deformable syn only (1 stage)\
            - b: rigid + affine + deformable b-spline syn (3 stages)\
            - br: rigid + deformable b-spline syn (2 stages)\
            - bo: deformable b-spline syn only (1 stage).
        radius: Radius for cross correlation metric used during SyN stage\
            (default = 4).
        spline_distance: Spline distance for deformable B-spline SyN transform\
            (default = 26).
        gradient_step: Gradient step size for SyN and B-spline SyN (default =\
            0.1).
        masks: Mask(s) for the fixed image space, or for the fixed and moving\
            image space in the format 'fixedMask,MovingMask'. Use -x once to\
            specify mask(s) to be used for all stages or use -x for each 'stage'\
            (cf -t option). If no mask is to be used for a particular stage, the\
            keyword 'NULL' should be used in place of file names.
        precision_type: Precision type (default = 'd'). Options:\
            - f: float\
            - d: double.
        use_histogram_matching: Use histogram matching (default = 0). Options:\
            - 0: false\
            - 1: true.
        use_repro_mode: Use 'repro' mode for exact reproducibility of output.\
            Uses GC metric for linear stages and a fixed random seed (default = 0).\
            Options:\
            - 0: false\
            - 1: true.
        collapse_output_transforms: Collapse output transforms (default = 1).\
            Options:\
            - 0: false\
            - 1: true.
        random_seed: Fix random seed to an int value.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsRegistrationSyN.sh",
        "image_dimension": image_dimension,
        "fixed_image": fixed_image,
        "moving_image": moving_image,
        "output_prefix": output_prefix,
    }
    if threads is not None:
        params["threads"] = threads
    if initial_transform is not None:
        params["initial_transform"] = initial_transform
    if transform_type is not None:
        params["transform_type"] = transform_type
    if radius is not None:
        params["radius"] = radius
    if spline_distance is not None:
        params["spline_distance"] = spline_distance
    if gradient_step is not None:
        params["gradient_step"] = gradient_step
    if masks is not None:
        params["masks"] = masks
    if precision_type is not None:
        params["precision_type"] = precision_type
    if use_histogram_matching is not None:
        params["use_histogram_matching"] = use_histogram_matching
    if use_repro_mode is not None:
        params["use_repro_mode"] = use_repro_mode
    if collapse_output_transforms is not None:
        params["collapse_output_transforms"] = collapse_output_transforms
    if random_seed is not None:
        params["random_seed"] = random_seed
    return params


def ants_registration_sy_n_sh_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsRegistrationSyNShParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `typing.Literal[2, 3]`')
    if params["image_dimension"] not in [2, 3]:
        raise StyxValidationError("Parameter `image_dimension` must be one of [2, 3]")
    if params.get("fixed_image", None) is None:
        raise StyxValidationError("`fixed_image` must not be None")
    if not isinstance(params["fixed_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`fixed_image` has the wrong type: Received `{type(params.get("fixed_image", None))}` expected `InputPathType`')
    if params.get("moving_image", None) is None:
        raise StyxValidationError("`moving_image` must not be None")
    if not isinstance(params["moving_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`moving_image` has the wrong type: Received `{type(params.get("moving_image", None))}` expected `InputPathType`')
    if params.get("output_prefix", None) is None:
        raise StyxValidationError("`output_prefix` must not be None")
    if not isinstance(params["output_prefix"], str):
        raise StyxValidationError(f'`output_prefix` has the wrong type: Received `{type(params.get("output_prefix", None))}` expected `str`')
    if params.get("threads", None) is not None:
        if not isinstance(params["threads"], int):
            raise StyxValidationError(f'`threads` has the wrong type: Received `{type(params.get("threads", None))}` expected `int | None`')
    if params.get("initial_transform", None) is not None:
        if not isinstance(params["initial_transform"], list):
            raise StyxValidationError(f'`initial_transform` has the wrong type: Received `{type(params.get("initial_transform", None))}` expected `list[str] | None`')
        for e in params["initial_transform"]:
            if not isinstance(e, str):
                raise StyxValidationError(f'`initial_transform` has the wrong type: Received `{type(params.get("initial_transform", None))}` expected `list[str] | None`')
    if params.get("transform_type", None) is not None:
        if not isinstance(params["transform_type"], str):
            raise StyxValidationError(f'`transform_type` has the wrong type: Received `{type(params.get("transform_type", None))}` expected `typing.Literal["t", "r", "a", "s", "sr", "so", "b", "br", "bo"] | None`')
        if params["transform_type"] not in ["t", "r", "a", "s", "sr", "so", "b", "br", "bo"]:
            raise StyxValidationError("Parameter `transform_type` must be one of [\"t\", \"r\", \"a\", \"s\", \"sr\", \"so\", \"b\", \"br\", \"bo\"]")
    if params.get("radius", None) is not None:
        if not isinstance(params["radius"], int):
            raise StyxValidationError(f'`radius` has the wrong type: Received `{type(params.get("radius", None))}` expected `int | None`')
    if params.get("spline_distance", None) is not None:
        if not isinstance(params["spline_distance"], int):
            raise StyxValidationError(f'`spline_distance` has the wrong type: Received `{type(params.get("spline_distance", None))}` expected `int | None`')
    if params.get("gradient_step", None) is not None:
        if not isinstance(params["gradient_step"], (float, int)):
            raise StyxValidationError(f'`gradient_step` has the wrong type: Received `{type(params.get("gradient_step", None))}` expected `float | None`')
    if params.get("masks", None) is not None:
        if not isinstance(params["masks"], str):
            raise StyxValidationError(f'`masks` has the wrong type: Received `{type(params.get("masks", None))}` expected `str | None`')
    if params.get("precision_type", None) is not None:
        if not isinstance(params["precision_type"], str):
            raise StyxValidationError(f'`precision_type` has the wrong type: Received `{type(params.get("precision_type", None))}` expected `typing.Literal["f", "d"] | None`')
        if params["precision_type"] not in ["f", "d"]:
            raise StyxValidationError("Parameter `precision_type` must be one of [\"f\", \"d\"]")
    if params.get("use_histogram_matching", None) is not None:
        if not isinstance(params["use_histogram_matching"], bool):
            raise StyxValidationError(f'`use_histogram_matching` has the wrong type: Received `{type(params.get("use_histogram_matching", None))}` expected `bool | None`')
    if params.get("use_repro_mode", None) is not None:
        if not isinstance(params["use_repro_mode"], bool):
            raise StyxValidationError(f'`use_repro_mode` has the wrong type: Received `{type(params.get("use_repro_mode", None))}` expected `bool | None`')
    if params.get("collapse_output_transforms", None) is not None:
        if not isinstance(params["collapse_output_transforms"], bool):
            raise StyxValidationError(f'`collapse_output_transforms` has the wrong type: Received `{type(params.get("collapse_output_transforms", None))}` expected `bool | None`')
    if params.get("random_seed", None) is not None:
        if not isinstance(params["random_seed"], int):
            raise StyxValidationError(f'`random_seed` has the wrong type: Received `{type(params.get("random_seed", None))}` expected `int | None`')


def ants_registration_sy_n_sh_cargs(
    params: AntsRegistrationSyNShParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsRegistrationSyN.sh")
    cargs.extend([
        "-d",
        str(params.get("image_dimension", None))
    ])
    cargs.extend([
        "-f",
        execution.input_file(params.get("fixed_image", None))
    ])
    cargs.extend([
        "-m",
        execution.input_file(params.get("moving_image", None))
    ])
    cargs.extend([
        "-o",
        params.get("output_prefix", None)
    ])
    if params.get("threads", None) is not None:
        cargs.extend([
            "-n",
            str(params.get("threads", None))
        ])
    if params.get("initial_transform", None) is not None:
        cargs.extend([
            "-i",
            "[" + ",".join(params.get("initial_transform", None)) + "]"
        ])
    if params.get("transform_type", None) is not None:
        cargs.extend([
            "-t",
            params.get("transform_type", None)
        ])
    if params.get("radius", None) is not None:
        cargs.extend([
            "-r",
            str(params.get("radius", None))
        ])
    if params.get("spline_distance", None) is not None:
        cargs.extend([
            "-s",
            str(params.get("spline_distance", None))
        ])
    if params.get("gradient_step", None) is not None:
        cargs.extend([
            "-g",
            str(params.get("gradient_step", None))
        ])
    if params.get("masks", None) is not None:
        cargs.extend([
            "-x",
            params.get("masks", None)
        ])
    if params.get("precision_type", None) is not None:
        cargs.extend([
            "-p",
            params.get("precision_type", None)
        ])
    if params.get("use_histogram_matching", None) is not None:
        cargs.extend([
            "-j",
            ("1" if params.get("use_histogram_matching", None) else "0")
        ])
    if params.get("use_repro_mode", None) is not None:
        cargs.extend([
            "-y",
            ("1" if params.get("use_repro_mode", None) else "0")
        ])
    if params.get("collapse_output_transforms", None) is not None:
        cargs.extend([
            "-z",
            ("1" if params.get("collapse_output_transforms", None) else "0")
        ])
    if params.get("random_seed", None) is not None:
        cargs.extend([
            "-e",
            str(params.get("random_seed", None))
        ])
    return cargs


def ants_registration_sy_n_sh_outputs(
    params: AntsRegistrationSyNShParamsDict,
    execution: Execution,
) -> AntsRegistrationSyNShOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsRegistrationSyNShOutputs(
        root=execution.output_file("."),
        affine_transform=execution.output_file(params.get("output_prefix", None) + "0GenericAffine.mat"),
        inverse_warp=execution.output_file(params.get("output_prefix", None) + "1InverseWarp.nii.gz"),
        forward_warp=execution.output_file(params.get("output_prefix", None) + "1Warp.nii.gz"),
    )
    return ret


def ants_registration_sy_n_sh_execute(
    params: AntsRegistrationSyNShParamsDict,
    runner: Runner | None = None,
) -> AntsRegistrationSyNShOutputs:
    """
    antsRegistrationSyN.sh
    
    Script for simplified symmetric image registration using ANTs.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsRegistrationSyNShOutputs`).
    """
    ants_registration_sy_n_sh_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_REGISTRATION_SY_N_SH_METADATA)
    params = execution.params(params)
    cargs = ants_registration_sy_n_sh_cargs(params, execution)
    ret = ants_registration_sy_n_sh_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_registration_sy_n_sh(
    image_dimension: typing.Literal[2, 3],
    fixed_image: InputPathType,
    moving_image: InputPathType,
    output_prefix: str,
    threads: int | None = None,
    initial_transform: list[str] | None = None,
    transform_type: typing.Literal["t", "r", "a", "s", "sr", "so", "b", "br", "bo"] | None = None,
    radius: int | None = None,
    spline_distance: int | None = None,
    gradient_step: float | None = None,
    masks: str | None = None,
    precision_type: typing.Literal["f", "d"] | None = None,
    use_histogram_matching: bool | None = None,
    use_repro_mode: bool | None = None,
    collapse_output_transforms: bool | None = None,
    random_seed: int | None = None,
    runner: Runner | None = None,
) -> AntsRegistrationSyNShOutputs:
    """
    antsRegistrationSyN.sh
    
    Script for simplified symmetric image registration using ANTs.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: Image dimension: 2 or 3 (for 2 or 3-dimensional\
            registration of a single volume).
        fixed_image: Fixed image(s) or source image(s) or reference image(s).
        moving_image: Moving image(s) or target image(s).
        output_prefix: A prefix that is prepended to all output files.
        threads: Number of threads (default =\
            ITK_GLOBAL_DEFAULT_NUMBER_OF_THREADS if defined, otherwise 1).
        initial_transform: Initial transform(s) --- order specified on the\
            command line matters.
        transform_type: Transform type (default = 's'). Options:\
            - t: translation (1 stage)\
            - r: rigid (1 stage)\
            - a: rigid + affine (2 stages)\
            - s: rigid + affine + deformable syn (3 stages)\
            - sr: rigid + deformable syn (2 stages)\
            - so: deformable syn only (1 stage)\
            - b: rigid + affine + deformable b-spline syn (3 stages)\
            - br: rigid + deformable b-spline syn (2 stages)\
            - bo: deformable b-spline syn only (1 stage).
        radius: Radius for cross correlation metric used during SyN stage\
            (default = 4).
        spline_distance: Spline distance for deformable B-spline SyN transform\
            (default = 26).
        gradient_step: Gradient step size for SyN and B-spline SyN (default =\
            0.1).
        masks: Mask(s) for the fixed image space, or for the fixed and moving\
            image space in the format 'fixedMask,MovingMask'. Use -x once to\
            specify mask(s) to be used for all stages or use -x for each 'stage'\
            (cf -t option). If no mask is to be used for a particular stage, the\
            keyword 'NULL' should be used in place of file names.
        precision_type: Precision type (default = 'd'). Options:\
            - f: float\
            - d: double.
        use_histogram_matching: Use histogram matching (default = 0). Options:\
            - 0: false\
            - 1: true.
        use_repro_mode: Use 'repro' mode for exact reproducibility of output.\
            Uses GC metric for linear stages and a fixed random seed (default = 0).\
            Options:\
            - 0: false\
            - 1: true.
        collapse_output_transforms: Collapse output transforms (default = 1).\
            Options:\
            - 0: false\
            - 1: true.
        random_seed: Fix random seed to an int value.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsRegistrationSyNShOutputs`).
    """
    params = ants_registration_sy_n_sh_params(
        image_dimension=image_dimension,
        fixed_image=fixed_image,
        moving_image=moving_image,
        output_prefix=output_prefix,
        threads=threads,
        initial_transform=initial_transform,
        transform_type=transform_type,
        radius=radius,
        spline_distance=spline_distance,
        gradient_step=gradient_step,
        masks=masks,
        precision_type=precision_type,
        use_histogram_matching=use_histogram_matching,
        use_repro_mode=use_repro_mode,
        collapse_output_transforms=collapse_output_transforms,
        random_seed=random_seed,
    )
    return ants_registration_sy_n_sh_execute(params, runner)


__all__ = [
    "ANTS_REGISTRATION_SY_N_SH_METADATA",
    "AntsRegistrationSyNShOutputs",
    "AntsRegistrationSyNShParamsDict",
    "AntsRegistrationSyNShParamsDictTagged",
    "ants_registration_sy_n_sh",
    "ants_registration_sy_n_sh_execute",
    "ants_registration_sy_n_sh_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

WARP_TIME_SERIES_IMAGE_MULTI_TRANSFORM_METADATA = Metadata(
    id="231dae6842dbb185b4794c2227e0956c3df99895.boutiques",
    name="WarpTimeSeriesImageMultiTransform",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_WarpTimeSeriesImageMultiTransformParamsDictNoTag = typing.TypedDict('_WarpTimeSeriesImageMultiTransformParamsDictNoTag', {
    "image_dimension": typing.Literal[3, 4],
    "moving_image": InputPathType,
    "output_image": str,
    "reference_image": InputPathType,
    "transforms": list[str],
    "interpolation": typing.NotRequired[typing.Literal["NearestNeighbor", "BSpline"] | None],
})
WarpTimeSeriesImageMultiTransformParamsDictTagged = typing.TypedDict('WarpTimeSeriesImageMultiTransformParamsDictTagged', {
    "@type": typing.Literal["ants/WarpTimeSeriesImageMultiTransform"],
    "image_dimension": typing.Literal[3, 4],
    "moving_image": InputPathType,
    "output_image": str,
    "reference_image": InputPathType,
    "transforms": list[str],
    "interpolation": typing.NotRequired[typing.Literal["NearestNeighbor", "BSpline"] | None],
})
WarpTimeSeriesImageMultiTransformParamsDict = _WarpTimeSeriesImageMultiTransformParamsDictNoTag | WarpTimeSeriesImageMultiTransformParamsDictTagged


class WarpTimeSeriesImageMultiTransformOutputs(typing.NamedTuple):
    """
    Output object returned when calling `WarpTimeSeriesImageMultiTransformParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_result: OutputPathType
    """The transformed image that is saved as output."""


def warp_time_series_image_multi_transform_params(
    image_dimension: typing.Literal[3, 4],
    moving_image: InputPathType,
    output_image: str,
    reference_image: InputPathType,
    transforms: list[str],
    interpolation: typing.Literal["NearestNeighbor", "BSpline"] | None = None,
) -> WarpTimeSeriesImageMultiTransformParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: The dimensionality of the input images (3D or 4D).
        moving_image: The image to which the transformation will be applied. It\
            can be a 3D image with vector voxels or a 4D image with scalar voxels.
        output_image: The output image after transformation. It is resampled\
            based on the reference image domain.
        reference_image: The reference image that defines the space into which\
            the input image will be warped.
        transforms: A list of transformation files, such as affine\
            transformation matrices and warps, applied in sequence.
        interpolation: Specifies the type of interpolation to use: Nearest\
            Neighbor or 3rd order B-Spline.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/WarpTimeSeriesImageMultiTransform",
        "image_dimension": image_dimension,
        "moving_image": moving_image,
        "output_image": output_image,
        "reference_image": reference_image,
        "transforms": transforms,
    }
    if interpolation is not None:
        params["interpolation"] = interpolation
    return params


def warp_time_series_image_multi_transform_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `WarpTimeSeriesImageMultiTransformParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `typing.Literal[3, 4]`')
    if params["image_dimension"] not in [3, 4]:
        raise StyxValidationError("Parameter `image_dimension` must be one of [3, 4]")
    if params.get("moving_image", None) is None:
        raise StyxValidationError("`moving_image` must not be None")
    if not isinstance(params["moving_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`moving_image` has the wrong type: Received `{type(params.get("moving_image", None))}` expected `InputPathType`')
    if params.get("output_image", None) is None:
        raise StyxValidationError("`output_image` must not be None")
    if not isinstance(params["output_image"], str):
        raise StyxValidationError(f'`output_image` has the wrong type: Received `{type(params.get("output_image", None))}` expected `str`')
    if params.get("reference_image", None) is None:
        raise StyxValidationError("`reference_image` must not be None")
    if not isinstance(params["reference_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`reference_image` has the wrong type: Received `{type(params.get("reference_image", None))}` expected `InputPathType`')
    if params.get("transforms", None) is None:
        raise StyxValidationError("`transforms` must not be None")
    if not isinstance(params["transforms"], list):
        raise StyxValidationError(f'`transforms` has the wrong type: Received `{type(params.get("transforms", None))}` expected `list[str]`')
    for e in params["transforms"]:
        if not isinstance(e, str):
            raise StyxValidationError(f'`transforms` has the wrong type: Received `{type(params.get("transforms", None))}` expected `list[str]`')
    if params.get("interpolation", None) is not None:
        if not isinstance(params["interpolation"], str):
            raise StyxValidationError(f'`interpolation` has the wrong type: Received `{type(params.get("interpolation", None))}` expected `typing.Literal["NearestNeighbor", "BSpline"] | None`')
        if params["interpolation"] not in ["NearestNeighbor", "BSpline"]:
            raise StyxValidationError("Parameter `interpolation` must be one of [\"NearestNeighbor\", \"BSpline\"]")


def warp_time_series_image_multi_transform_cargs(
    params: WarpTimeSeriesImageMultiTransformParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("WarpTimeSeriesImageMultiTransform")
    cargs.append(str(params.get("image_dimension", None)))
    cargs.append(execution.input_file(params.get("moving_image", None)))
    cargs.append(params.get("output_image", None))
    cargs.extend([
        "-R",
        execution.input_file(params.get("reference_image", None))
    ])
    cargs.extend(params.get("transforms", None))
    if params.get("interpolation", None) is not None:
        cargs.append(params.get("interpolation", None))
    return cargs


def warp_time_series_image_multi_transform_outputs(
    params: WarpTimeSeriesImageMultiTransformParamsDict,
    execution: Execution,
) -> WarpTimeSeriesImageMultiTransformOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = WarpTimeSeriesImageMultiTransformOutputs(
        root=execution.output_file("."),
        output_image_result=execution.output_file(params.get("output_image", None)),
    )
    return ret


def warp_time_series_image_multi_transform_execute(
    params: WarpTimeSeriesImageMultiTransformParamsDict,
    runner: Runner | None = None,
) -> WarpTimeSeriesImageMultiTransformOutputs:
    """
    WarpTimeSeriesImageMultiTransform
    
    WarpTimeSeriesImageMultiTransform is a tool used to apply a series of
    transformations to a time series image, either forward or reverse, using
    affine transforms and warps.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `WarpTimeSeriesImageMultiTransformOutputs`).
    """
    warp_time_series_image_multi_transform_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(WARP_TIME_SERIES_IMAGE_MULTI_TRANSFORM_METADATA)
    params = execution.params(params)
    cargs = warp_time_series_image_multi_transform_cargs(params, execution)
    ret = warp_time_series_image_multi_transform_outputs(params, execution)
    execution.run(cargs)
    return ret


def warp_time_series_image_multi_transform(
    image_dimension: typing.Literal[3, 4],
    moving_image: InputPathType,
    output_image: str,
    reference_image: InputPathType,
    transforms: list[str],
    interpolation: typing.Literal["NearestNeighbor", "BSpline"] | None = None,
    runner: Runner | None = None,
) -> WarpTimeSeriesImageMultiTransformOutputs:
    """
    WarpTimeSeriesImageMultiTransform
    
    WarpTimeSeriesImageMultiTransform is a tool used to apply a series of
    transformations to a time series image, either forward or reverse, using
    affine transforms and warps.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: The dimensionality of the input images (3D or 4D).
        moving_image: The image to which the transformation will be applied. It\
            can be a 3D image with vector voxels or a 4D image with scalar voxels.
        output_image: The output image after transformation. It is resampled\
            based on the reference image domain.
        reference_image: The reference image that defines the space into which\
            the input image will be warped.
        transforms: A list of transformation files, such as affine\
            transformation matrices and warps, applied in sequence.
        interpolation: Specifies the type of interpolation to use: Nearest\
            Neighbor or 3rd order B-Spline.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `WarpTimeSeriesImageMultiTransformOutputs`).
    """
    params = warp_time_series_image_multi_transform_params(
        image_dimension=image_dimension,
        moving_image=moving_image,
        output_image=output_image,
        reference_image=reference_image,
        transforms=transforms,
        interpolation=interpolation,
    )
    return warp_time_series_image_multi_transform_execute(params, runner)


__all__ = [
    "WARP_TIME_SERIES_IMAGE_MULTI_TRANSFORM_METADATA",
    "WarpTimeSeriesImageMultiTransformOutputs",
    "WarpTimeSeriesImageMultiTransformParamsDict",
    "WarpTimeSeriesImageMultiTransformParamsDictTagged",
    "warp_time_series_image_multi_transform",
    "warp_time_series_image_multi_transform_execute",
    "warp_time_series_image_multi_transform_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

EXTRACT_REGION_FROM_IMAGE_BY_MASK_METADATA = Metadata(
    id="a771c6b15a68298a66132d0c551dee738e184a1d.boutiques",
    name="ExtractRegionFromImageByMask",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_ExtractRegionFromImageByMaskParamsDictNoTag = typing.TypedDict('_ExtractRegionFromImageByMaskParamsDictNoTag', {
    "image_dimension": int,
    "input_image": InputPathType,
    "output_image": InputPathType,
    "label_mask_image": InputPathType,
    "label": typing.NotRequired[int | None],
    "pad_radius": typing.NotRequired[int | None],
})
ExtractRegionFromImageByMaskParamsDictTagged = typing.TypedDict('ExtractRegionFromImageByMaskParamsDictTagged', {
    "@type": typing.Literal["ants/ExtractRegionFromImageByMask"],
    "image_dimension": int,
    "input_image": InputPathType,
    "output_image": InputPathType,
    "label_mask_image": InputPathType,
    "label": typing.NotRequired[int | None],
    "pad_radius": typing.NotRequired[int | None],
})
ExtractRegionFromImageByMaskParamsDict = _ExtractRegionFromImageByMaskParamsDictNoTag | ExtractRegionFromImageByMaskParamsDictTagged


class ExtractRegionFromImageByMaskOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ExtractRegionFromImageByMaskParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def extract_region_from_image_by_mask_params(
    image_dimension: int,
    input_image: InputPathType,
    output_image: InputPathType,
    label_mask_image: InputPathType,
    label: int | None = None,
    pad_radius: int | None = None,
) -> ExtractRegionFromImageByMaskParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image_dimension: Dimension of the input image.
        input_image: The input image from which a region will be extracted.
        output_image: The output image containing the extracted region.
        label_mask_image: The label mask image used to extract the bounding\
            box.
        label: The label value used to extract the sub-region.
        pad_radius: Optional padding radius to be added around the bounding\
            box.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/ExtractRegionFromImageByMask",
        "image_dimension": image_dimension,
        "input_image": input_image,
        "output_image": output_image,
        "label_mask_image": label_mask_image,
    }
    if label is not None:
        params["label"] = label
    if pad_radius is not None:
        params["pad_radius"] = pad_radius
    return params


def extract_region_from_image_by_mask_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ExtractRegionFromImageByMaskParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimension", None) is None:
        raise StyxValidationError("`image_dimension` must not be None")
    if not isinstance(params["image_dimension"], int):
        raise StyxValidationError(f'`image_dimension` has the wrong type: Received `{type(params.get("image_dimension", None))}` expected `int`')
    if params.get("input_image", None) is None:
        raise StyxValidationError("`input_image` must not be None")
    if not isinstance(params["input_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_image` has the wrong type: Received `{type(params.get("input_image", None))}` expected `InputPathType`')
    if params.get("output_image", None) is None:
        raise StyxValidationError("`output_image` must not be None")
    if not isinstance(params["output_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`output_image` has the wrong type: Received `{type(params.get("output_image", None))}` expected `InputPathType`')
    if params.get("label_mask_image", None) is None:
        raise StyxValidationError("`label_mask_image` must not be None")
    if not isinstance(params["label_mask_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label_mask_image` has the wrong type: Received `{type(params.get("label_mask_image", None))}` expected `InputPathType`')
    if params.get("label", None) is not None:
        if not isinstance(params["label"], int):
            raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `int | None`')
    if params.get("pad_radius", None) is not None:
        if not isinstance(params["pad_radius"], int):
            raise StyxValidationError(f'`pad_radius` has the wrong type: Received `{type(params.get("pad_radius", None))}` expected `int | None`')


def extract_region_from_image_by_mask_cargs(
    params: ExtractRegionFromImageByMaskParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ExtractRegionFromImageByMask")
    cargs.append(str(params.get("image_dimension", None)))
    cargs.append(execution.input_file(params.get("input_image", None)))
    cargs.append(execution.input_file(params.get("output_image", None)))
    cargs.append(execution.input_file(params.get("label_mask_image", None)))
    if params.get("label", None) is not None:
        cargs.append(str(params.get("label", None)))
    if params.get("pad_radius", None) is not None:
        cargs.append(str(params.get("pad_radius", None)))
    return cargs


def extract_region_from_image_by_mask_outputs(
    params: ExtractRegionFromImageByMaskParamsDict,
    execution: Execution,
) -> ExtractRegionFromImageByMaskOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ExtractRegionFromImageByMaskOutputs(
        root=execution.output_file("."),
    )
    return ret


def extract_region_from_image_by_mask_execute(
    params: ExtractRegionFromImageByMaskParamsDict,
    runner: Runner | None = None,
) -> ExtractRegionFromImageByMaskOutputs:
    """
    ExtractRegionFromImageByMask
    
    Extract a sub-region from an image using the bounding box from a label
    image, with an optional padding radius.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ExtractRegionFromImageByMaskOutputs`).
    """
    extract_region_from_image_by_mask_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(EXTRACT_REGION_FROM_IMAGE_BY_MASK_METADATA)
    params = execution.params(params)
    cargs = extract_region_from_image_by_mask_cargs(params, execution)
    ret = extract_region_from_image_by_mask_outputs(params, execution)
    execution.run(cargs)
    return ret


def extract_region_from_image_by_mask(
    image_dimension: int,
    input_image: InputPathType,
    output_image: InputPathType,
    label_mask_image: InputPathType,
    label: int | None = None,
    pad_radius: int | None = None,
    runner: Runner | None = None,
) -> ExtractRegionFromImageByMaskOutputs:
    """
    ExtractRegionFromImageByMask
    
    Extract a sub-region from an image using the bounding box from a label
    image, with an optional padding radius.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: Dimension of the input image.
        input_image: The input image from which a region will be extracted.
        output_image: The output image containing the extracted region.
        label_mask_image: The label mask image used to extract the bounding\
            box.
        label: The label value used to extract the sub-region.
        pad_radius: Optional padding radius to be added around the bounding\
            box.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ExtractRegionFromImageByMaskOutputs`).
    """
    params = extract_region_from_image_by_mask_params(
        image_dimension=image_dimension,
        input_image=input_image,
        output_image=output_image,
        label_mask_image=label_mask_image,
        label=label,
        pad_radius=pad_radius,
    )
    return extract_region_from_image_by_mask_execute(params, runner)


__all__ = [
    "EXTRACT_REGION_FROM_IMAGE_BY_MASK_METADATA",
    "ExtractRegionFromImageByMaskOutputs",
    "ExtractRegionFromImageByMaskParamsDict",
    "ExtractRegionFromImageByMaskParamsDictTagged",
    "extract_region_from_image_by_mask",
    "extract_region_from_image_by_mask_execute",
    "extract_region_from_image_by_mask_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTSJACOBIAN_METADATA = Metadata(
    id="d3e1548784785ff25ca205e88e434be38d8b149c.boutiques",
    name="ANTSJacobian",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsjacobianParamsDictNoTag = typing.TypedDict('_AntsjacobianParamsDictNoTag', {
    "imagedim": int,
    "gwarp": InputPathType,
    "outfile": str,
    "uselog": int,
    "maskfn": InputPathType,
    "normbytotalbool": int,
    "projectionvector": typing.NotRequired[str | None],
})
AntsjacobianParamsDictTagged = typing.TypedDict('AntsjacobianParamsDictTagged', {
    "@type": typing.Literal["ants/ANTSJacobian"],
    "imagedim": int,
    "gwarp": InputPathType,
    "outfile": str,
    "uselog": int,
    "maskfn": InputPathType,
    "normbytotalbool": int,
    "projectionvector": typing.NotRequired[str | None],
})
AntsjacobianParamsDict = _AntsjacobianParamsDictNoTag | AntsjacobianParamsDictTagged


class AntsjacobianOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsjacobianParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    jacobian_output: OutputPathType
    """Output file containing the Jacobian determinant information."""


def antsjacobian_params(
    imagedim: int,
    gwarp: InputPathType,
    outfile: str,
    uselog: int,
    maskfn: InputPathType,
    normbytotalbool: int,
    projectionvector: str | None = None,
) -> AntsjacobianParamsDictTagged:
    """
    Build parameters.
    
    Args:
        imagedim: The dimensionality of the input image.
        gwarp: The input warp image.
        outfile: The prefix for the output files.
        uselog: Whether to use logarithm in computation.
        maskfn: Mask file used in the computation.
        normbytotalbool: Normalize the Jacobian by the total in the mask. Use\
            this to adjust for head size.
        projectionvector: Projects the warp along the specified direction. Do\
            not add this option if no projection is desired.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/ANTSJacobian",
        "imagedim": imagedim,
        "gwarp": gwarp,
        "outfile": outfile,
        "uselog": uselog,
        "maskfn": maskfn,
        "normbytotalbool": normbytotalbool,
    }
    if projectionvector is not None:
        params["projectionvector"] = projectionvector
    return params


def antsjacobian_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsjacobianParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("imagedim", None) is None:
        raise StyxValidationError("`imagedim` must not be None")
    if not isinstance(params["imagedim"], int):
        raise StyxValidationError(f'`imagedim` has the wrong type: Received `{type(params.get("imagedim", None))}` expected `int`')
    if params.get("gwarp", None) is None:
        raise StyxValidationError("`gwarp` must not be None")
    if not isinstance(params["gwarp"], (pathlib.Path, str)):
        raise StyxValidationError(f'`gwarp` has the wrong type: Received `{type(params.get("gwarp", None))}` expected `InputPathType`')
    if params.get("outfile", None) is None:
        raise StyxValidationError("`outfile` must not be None")
    if not isinstance(params["outfile"], str):
        raise StyxValidationError(f'`outfile` has the wrong type: Received `{type(params.get("outfile", None))}` expected `str`')
    if params.get("uselog", None) is None:
        raise StyxValidationError("`uselog` must not be None")
    if not isinstance(params["uselog"], int):
        raise StyxValidationError(f'`uselog` has the wrong type: Received `{type(params.get("uselog", None))}` expected `int`')
    if params.get("maskfn", None) is None:
        raise StyxValidationError("`maskfn` must not be None")
    if not isinstance(params["maskfn"], (pathlib.Path, str)):
        raise StyxValidationError(f'`maskfn` has the wrong type: Received `{type(params.get("maskfn", None))}` expected `InputPathType`')
    if params.get("normbytotalbool", None) is None:
        raise StyxValidationError("`normbytotalbool` must not be None")
    if not isinstance(params["normbytotalbool"], int):
        raise StyxValidationError(f'`normbytotalbool` has the wrong type: Received `{type(params.get("normbytotalbool", None))}` expected `int`')
    if params.get("projectionvector", None) is not None:
        if not isinstance(params["projectionvector"], str):
            raise StyxValidationError(f'`projectionvector` has the wrong type: Received `{type(params.get("projectionvector", None))}` expected `str | None`')


def antsjacobian_cargs(
    params: AntsjacobianParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ANTSJacobian")
    cargs.append(str(params.get("imagedim", None)))
    cargs.append(execution.input_file(params.get("gwarp", None)))
    cargs.append(params.get("outfile", None))
    cargs.append(str(params.get("uselog", None)))
    cargs.append(execution.input_file(params.get("maskfn", None)))
    cargs.append(str(params.get("normbytotalbool", None)))
    if params.get("projectionvector", None) is not None:
        cargs.append(params.get("projectionvector", None))
    return cargs


def antsjacobian_outputs(
    params: AntsjacobianParamsDict,
    execution: Execution,
) -> AntsjacobianOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsjacobianOutputs(
        root=execution.output_file("."),
        jacobian_output=execution.output_file(params.get("outfile", None) + "Jacobian.nii.gz"),
    )
    return ret


def antsjacobian_execute(
    params: AntsjacobianParamsDict,
    runner: Runner | None = None,
) -> AntsjacobianOutputs:
    """
    ANTSJacobian
    
    Calculate the Jacobian determinant of a transformation using ANTs. WARNING:
    ANTSJacobian may not be working correctly; see
    CreateJacobianDeterminantImage for an alternative method.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsjacobianOutputs`).
    """
    antsjacobian_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTSJACOBIAN_METADATA)
    params = execution.params(params)
    cargs = antsjacobian_cargs(params, execution)
    ret = antsjacobian_outputs(params, execution)
    execution.run(cargs)
    return ret


def antsjacobian(
    imagedim: int,
    gwarp: InputPathType,
    outfile: str,
    uselog: int,
    maskfn: InputPathType,
    normbytotalbool: int,
    projectionvector: str | None = None,
    runner: Runner | None = None,
) -> AntsjacobianOutputs:
    """
    ANTSJacobian
    
    Calculate the Jacobian determinant of a transformation using ANTs. WARNING:
    ANTSJacobian may not be working correctly; see
    CreateJacobianDeterminantImage for an alternative method.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        imagedim: The dimensionality of the input image.
        gwarp: The input warp image.
        outfile: The prefix for the output files.
        uselog: Whether to use logarithm in computation.
        maskfn: Mask file used in the computation.
        normbytotalbool: Normalize the Jacobian by the total in the mask. Use\
            this to adjust for head size.
        projectionvector: Projects the warp along the specified direction. Do\
            not add this option if no projection is desired.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsjacobianOutputs`).
    """
    params = antsjacobian_params(
        imagedim=imagedim,
        gwarp=gwarp,
        outfile=outfile,
        uselog=uselog,
        maskfn=maskfn,
        normbytotalbool=normbytotalbool,
        projectionvector=projectionvector,
    )
    return antsjacobian_execute(params, runner)


__all__ = [
    "ANTSJACOBIAN_METADATA",
    "AntsjacobianOutputs",
    "AntsjacobianParamsDict",
    "AntsjacobianParamsDictTagged",
    "antsjacobian",
    "antsjacobian_execute",
    "antsjacobian_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SCCAN_METADATA = Metadata(
    id="231aefef222d26c3ea09014dbf8d6f3d8b7ad5cf.boutiques",
    name="sccan",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_SccanParamsDictNoTag = typing.TypedDict('_SccanParamsDictNoTag', {
    "output": typing.NotRequired[str | None],
    "n_permutations": typing.NotRequired[int | None],
    "smoother": typing.NotRequired[int | None],
    "row_sparseness": typing.NotRequired[int | None],
    "iterations": typing.NotRequired[int | None],
    "n_eigenvectors": typing.NotRequired[int | None],
    "robustify": typing.NotRequired[int | None],
    "covering": typing.NotRequired[int | None],
    "uselong": typing.NotRequired[int | None],
    "l1": typing.NotRequired[float | None],
    "pclusterthresh": typing.NotRequired[float | None],
    "qclusterthresh": typing.NotRequired[float | None],
    "ridge_cca": typing.NotRequired[float | None],
    "initialization": typing.NotRequired[str | None],
    "initialization2": typing.NotRequired[str | None],
    "mask": typing.NotRequired[InputPathType | None],
    "mask2": typing.NotRequired[InputPathType | None],
    "partial_scca_option": typing.NotRequired[str | None],
    "prior_weight": typing.NotRequired[float | None],
    "get_small": typing.NotRequired[float | None],
    "verbose": typing.NotRequired[float | None],
    "imageset_to_matrix": typing.NotRequired[str | None],
    "timeseriesimage_to_matrix": typing.NotRequired[str | None],
    "vector_to_image": typing.NotRequired[str | None],
    "imageset_to_projections": typing.NotRequired[str | None],
    "scca": typing.NotRequired[str | None],
    "svd": typing.NotRequired[str | None],
})
SccanParamsDictTagged = typing.TypedDict('SccanParamsDictTagged', {
    "@type": typing.Literal["ants/sccan"],
    "output": typing.NotRequired[str | None],
    "n_permutations": typing.NotRequired[int | None],
    "smoother": typing.NotRequired[int | None],
    "row_sparseness": typing.NotRequired[int | None],
    "iterations": typing.NotRequired[int | None],
    "n_eigenvectors": typing.NotRequired[int | None],
    "robustify": typing.NotRequired[int | None],
    "covering": typing.NotRequired[int | None],
    "uselong": typing.NotRequired[int | None],
    "l1": typing.NotRequired[float | None],
    "pclusterthresh": typing.NotRequired[float | None],
    "qclusterthresh": typing.NotRequired[float | None],
    "ridge_cca": typing.NotRequired[float | None],
    "initialization": typing.NotRequired[str | None],
    "initialization2": typing.NotRequired[str | None],
    "mask": typing.NotRequired[InputPathType | None],
    "mask2": typing.NotRequired[InputPathType | None],
    "partial_scca_option": typing.NotRequired[str | None],
    "prior_weight": typing.NotRequired[float | None],
    "get_small": typing.NotRequired[float | None],
    "verbose": typing.NotRequired[float | None],
    "imageset_to_matrix": typing.NotRequired[str | None],
    "timeseriesimage_to_matrix": typing.NotRequired[str | None],
    "vector_to_image": typing.NotRequired[str | None],
    "imageset_to_projections": typing.NotRequired[str | None],
    "scca": typing.NotRequired[str | None],
    "svd": typing.NotRequired[str | None],
})
SccanParamsDict = _SccanParamsDictNoTag | SccanParamsDictTagged


class SccanOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SccanParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def sccan_params(
    output: str | None = None,
    n_permutations: int | None = None,
    smoother: int | None = None,
    row_sparseness: int | None = None,
    iterations: int | None = None,
    n_eigenvectors: int | None = None,
    robustify: int | None = None,
    covering: int | None = None,
    uselong: int | None = None,
    l1: float | None = None,
    pclusterthresh: float | None = None,
    qclusterthresh: float | None = None,
    ridge_cca: float | None = None,
    initialization: str | None = None,
    initialization2: str | None = None,
    mask: InputPathType | None = None,
    mask2: InputPathType | None = None,
    partial_scca_option: str | None = None,
    prior_weight: float | None = None,
    get_small: float | None = None,
    verbose: float | None = None,
    imageset_to_matrix: str | None = None,
    timeseriesimage_to_matrix: str | None = None,
    vector_to_image: str | None = None,
    imageset_to_projections: str | None = None,
    scca: str | None = None,
    svd: str | None = None,
) -> SccanParamsDictTagged:
    """
    Build parameters.
    
    Args:
        output: Output dependent on which option is called.
        n_permutations: Number of permutations to use in scca.
        smoother: Smoothing function for variates.
        row_sparseness: Row sparseness - if (+) then keep values (+) otherwise\
            allow +/- values --- always L1.
        iterations: Max iterations for scca optimization (min 20).
        n_eigenvectors: Number of eigenvectors to compute in scca/spca.
        robustify: Rank-based scca.
        covering: Try to make the decomposition cover the whole domain, if\
            possible.
        uselong: Use longitudinal formulation (> 0) or not (<= 0).
        l1: Use l1 (> 0) or l0 (< 0) penalty, also sets gradient step size.
        pclusterthresh: Cluster threshold on view P.
        qclusterthresh: Cluster threshold on view Q.
        ridge_cca: Ridge cca.
        initialization: Initialization file list for Eigenanatomy - must also\
            pass mask option.
        initialization2: Initialization file list for SCCAN-Eigenanatomy - must\
            also pass mask option.
        mask: Mask file for Eigenanatomy initialization.
        mask2: Mask file for Eigenanatomy initialization 2.
        partial_scca_option: Choices for pscca: PQ, PminusRQ, PQminusR,\
            PminusRQminusR.
        prior_weight: Scalar value weight on prior between 0 (prior is weak)\
            and 1 (prior is strong). Only engaged if initialization is used.
        get_small: Find smallest eigenvectors.
        verbose: Set whether output is verbose.
        imageset_to_matrix: Takes a list of image files names (one per line)\
            and converts it to a 2D matrix/image in binary or csv format.
        timeseriesimage_to_matrix: Takes a timeseries (4D) image and converts\
            it to a 2D matrix csv format.
        vector_to_image: Converts the 1st column vector in a csv file back to\
            an image.
        imageset_to_projections: Takes a list of image and projection files\
            names and writes them to a csv file.
        scca: Matrix-based scca operations for 2 and 3 views.
        svd: A sparse SVD implementation.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/sccan",
    }
    if output is not None:
        params["output"] = output
    if n_permutations is not None:
        params["n_permutations"] = n_permutations
    if smoother is not None:
        params["smoother"] = smoother
    if row_sparseness is not None:
        params["row_sparseness"] = row_sparseness
    if iterations is not None:
        params["iterations"] = iterations
    if n_eigenvectors is not None:
        params["n_eigenvectors"] = n_eigenvectors
    if robustify is not None:
        params["robustify"] = robustify
    if covering is not None:
        params["covering"] = covering
    if uselong is not None:
        params["uselong"] = uselong
    if l1 is not None:
        params["l1"] = l1
    if pclusterthresh is not None:
        params["pclusterthresh"] = pclusterthresh
    if qclusterthresh is not None:
        params["qclusterthresh"] = qclusterthresh
    if ridge_cca is not None:
        params["ridge_cca"] = ridge_cca
    if initialization is not None:
        params["initialization"] = initialization
    if initialization2 is not None:
        params["initialization2"] = initialization2
    if mask is not None:
        params["mask"] = mask
    if mask2 is not None:
        params["mask2"] = mask2
    if partial_scca_option is not None:
        params["partial_scca_option"] = partial_scca_option
    if prior_weight is not None:
        params["prior_weight"] = prior_weight
    if get_small is not None:
        params["get_small"] = get_small
    if verbose is not None:
        params["verbose"] = verbose
    if imageset_to_matrix is not None:
        params["imageset_to_matrix"] = imageset_to_matrix
    if timeseriesimage_to_matrix is not None:
        params["timeseriesimage_to_matrix"] = timeseriesimage_to_matrix
    if vector_to_image is not None:
        params["vector_to_image"] = vector_to_image
    if imageset_to_projections is not None:
        params["imageset_to_projections"] = imageset_to_projections
    if scca is not None:
        params["scca"] = scca
    if svd is not None:
        params["svd"] = svd
    return params


def sccan_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SccanParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("output", None) is not None:
        if not isinstance(params["output"], str):
            raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str | None`')
    if params.get("n_permutations", None) is not None:
        if not isinstance(params["n_permutations"], int):
            raise StyxValidationError(f'`n_permutations` has the wrong type: Received `{type(params.get("n_permutations", None))}` expected `int | None`')
    if params.get("smoother", None) is not None:
        if not isinstance(params["smoother"], int):
            raise StyxValidationError(f'`smoother` has the wrong type: Received `{type(params.get("smoother", None))}` expected `int | None`')
    if params.get("row_sparseness", None) is not None:
        if not isinstance(params["row_sparseness"], int):
            raise StyxValidationError(f'`row_sparseness` has the wrong type: Received `{type(params.get("row_sparseness", None))}` expected `int | None`')
    if params.get("iterations", None) is not None:
        if not isinstance(params["iterations"], int):
            raise StyxValidationError(f'`iterations` has the wrong type: Received `{type(params.get("iterations", None))}` expected `int | None`')
        if params["iterations"] < 20:
            raise StyxValidationError("Parameter `iterations` must be at least 20")
    if params.get("n_eigenvectors", None) is not None:
        if not isinstance(params["n_eigenvectors"], int):
            raise StyxValidationError(f'`n_eigenvectors` has the wrong type: Received `{type(params.get("n_eigenvectors", None))}` expected `int | None`')
    if params.get("robustify", None) is not None:
        if not isinstance(params["robustify"], int):
            raise StyxValidationError(f'`robustify` has the wrong type: Received `{type(params.get("robustify", None))}` expected `int | None`')
    if params.get("covering", None) is not None:
        if not isinstance(params["covering"], int):
            raise StyxValidationError(f'`covering` has the wrong type: Received `{type(params.get("covering", None))}` expected `int | None`')
    if params.get("uselong", None) is not None:
        if not isinstance(params["uselong"], int):
            raise StyxValidationError(f'`uselong` has the wrong type: Received `{type(params.get("uselong", None))}` expected `int | None`')
    if params.get("l1", None) is not None:
        if not isinstance(params["l1"], (float, int)):
            raise StyxValidationError(f'`l1` has the wrong type: Received `{type(params.get("l1", None))}` expected `float | None`')
    if params.get("pclusterthresh", None) is not None:
        if not isinstance(params["pclusterthresh"], (float, int)):
            raise StyxValidationError(f'`pclusterthresh` has the wrong type: Received `{type(params.get("pclusterthresh", None))}` expected `float | None`')
    if params.get("qclusterthresh", None) is not None:
        if not isinstance(params["qclusterthresh"], (float, int)):
            raise StyxValidationError(f'`qclusterthresh` has the wrong type: Received `{type(params.get("qclusterthresh", None))}` expected `float | None`')
    if params.get("ridge_cca", None) is not None:
        if not isinstance(params["ridge_cca"], (float, int)):
            raise StyxValidationError(f'`ridge_cca` has the wrong type: Received `{type(params.get("ridge_cca", None))}` expected `float | None`')
    if params.get("initialization", None) is not None:
        if not isinstance(params["initialization"], str):
            raise StyxValidationError(f'`initialization` has the wrong type: Received `{type(params.get("initialization", None))}` expected `str | None`')
    if params.get("initialization2", None) is not None:
        if not isinstance(params["initialization2"], str):
            raise StyxValidationError(f'`initialization2` has the wrong type: Received `{type(params.get("initialization2", None))}` expected `str | None`')
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')
    if params.get("mask2", None) is not None:
        if not isinstance(params["mask2"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask2` has the wrong type: Received `{type(params.get("mask2", None))}` expected `InputPathType | None`')
    if params.get("partial_scca_option", None) is not None:
        if not isinstance(params["partial_scca_option"], str):
            raise StyxValidationError(f'`partial_scca_option` has the wrong type: Received `{type(params.get("partial_scca_option", None))}` expected `str | None`')
    if params.get("prior_weight", None) is not None:
        if not isinstance(params["prior_weight"], (float, int)):
            raise StyxValidationError(f'`prior_weight` has the wrong type: Received `{type(params.get("prior_weight", None))}` expected `float | None`')
    if params.get("get_small", None) is not None:
        if not isinstance(params["get_small"], (float, int)):
            raise StyxValidationError(f'`get_small` has the wrong type: Received `{type(params.get("get_small", None))}` expected `float | None`')
    if params.get("verbose", None) is not None:
        if not isinstance(params["verbose"], (float, int)):
            raise StyxValidationError(f'`verbose` has the wrong type: Received `{type(params.get("verbose", None))}` expected `float | None`')
    if params.get("imageset_to_matrix", None) is not None:
        if not isinstance(params["imageset_to_matrix"], str):
            raise StyxValidationError(f'`imageset_to_matrix` has the wrong type: Received `{type(params.get("imageset_to_matrix", None))}` expected `str | None`')
    if params.get("timeseriesimage_to_matrix", None) is not None:
        if not isinstance(params["timeseriesimage_to_matrix"], str):
            raise StyxValidationError(f'`timeseriesimage_to_matrix` has the wrong type: Received `{type(params.get("timeseriesimage_to_matrix", None))}` expected `str | None`')
    if params.get("vector_to_image", None) is not None:
        if not isinstance(params["vector_to_image"], str):
            raise StyxValidationError(f'`vector_to_image` has the wrong type: Received `{type(params.get("vector_to_image", None))}` expected `str | None`')
    if params.get("imageset_to_projections", None) is not None:
        if not isinstance(params["imageset_to_projections"], str):
            raise StyxValidationError(f'`imageset_to_projections` has the wrong type: Received `{type(params.get("imageset_to_projections", None))}` expected `str | None`')
    if params.get("scca", None) is not None:
        if not isinstance(params["scca"], str):
            raise StyxValidationError(f'`scca` has the wrong type: Received `{type(params.get("scca", None))}` expected `str | None`')
    if params.get("svd", None) is not None:
        if not isinstance(params["svd"], str):
            raise StyxValidationError(f'`svd` has the wrong type: Received `{type(params.get("svd", None))}` expected `str | None`')


def sccan_cargs(
    params: SccanParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("sccan")
    if params.get("output", None) is not None:
        cargs.extend([
            "-o",
            params.get("output", None)
        ])
    if params.get("n_permutations", None) is not None:
        cargs.extend([
            "-p",
            str(params.get("n_permutations", None))
        ])
    if params.get("smoother", None) is not None:
        cargs.extend([
            "-s",
            str(params.get("smoother", None))
        ])
    if params.get("row_sparseness", None) is not None:
        cargs.extend([
            "-z",
            str(params.get("row_sparseness", None))
        ])
    if params.get("iterations", None) is not None:
        cargs.extend([
            "-i",
            str(params.get("iterations", None))
        ])
    if params.get("n_eigenvectors", None) is not None:
        cargs.extend([
            "-n",
            str(params.get("n_eigenvectors", None))
        ])
    if params.get("robustify", None) is not None:
        cargs.extend([
            "-r",
            str(params.get("robustify", None))
        ])
    if params.get("covering", None) is not None:
        cargs.extend([
            "-c",
            str(params.get("covering", None))
        ])
    if params.get("uselong", None) is not None:
        cargs.extend([
            "-g",
            str(params.get("uselong", None))
        ])
    if params.get("l1", None) is not None:
        cargs.extend([
            "-l",
            str(params.get("l1", None))
        ])
    if params.get("pclusterthresh", None) is not None:
        cargs.extend([
            "--PClusterThresh",
            str(params.get("pclusterthresh", None))
        ])
    if params.get("qclusterthresh", None) is not None:
        cargs.extend([
            "--QClusterThresh",
            str(params.get("qclusterthresh", None))
        ])
    if params.get("ridge_cca", None) is not None:
        cargs.extend([
            "-e",
            str(params.get("ridge_cca", None))
        ])
    if params.get("initialization", None) is not None:
        cargs.extend([
            "--initialization",
            params.get("initialization", None)
        ])
    if params.get("initialization2", None) is not None:
        cargs.extend([
            "--initialization2",
            params.get("initialization2", None)
        ])
    if params.get("mask", None) is not None:
        cargs.extend([
            "--mask",
            execution.input_file(params.get("mask", None))
        ])
    if params.get("mask2", None) is not None:
        cargs.extend([
            "--mask2",
            execution.input_file(params.get("mask2", None))
        ])
    if params.get("partial_scca_option", None) is not None:
        cargs.extend([
            "--partial-scca-option",
            params.get("partial_scca_option", None)
        ])
    if params.get("prior_weight", None) is not None:
        cargs.extend([
            "--prior-weight",
            str(params.get("prior_weight", None))
        ])
    if params.get("get_small", None) is not None:
        cargs.extend([
            "--get-small",
            str(params.get("get_small", None))
        ])
    if params.get("verbose", None) is not None:
        cargs.extend([
            "-v",
            str(params.get("verbose", None))
        ])
    if params.get("imageset_to_matrix", None) is not None:
        cargs.extend([
            "--imageset-to-matrix",
            params.get("imageset_to_matrix", None)
        ])
    if params.get("timeseriesimage_to_matrix", None) is not None:
        cargs.extend([
            "--timeseriesimage-to-matrix",
            params.get("timeseriesimage_to_matrix", None)
        ])
    if params.get("vector_to_image", None) is not None:
        cargs.extend([
            "--vector-to-image",
            params.get("vector_to_image", None)
        ])
    if params.get("imageset_to_projections", None) is not None:
        cargs.extend([
            "--imageset-to-projections",
            params.get("imageset_to_projections", None)
        ])
    if params.get("scca", None) is not None:
        cargs.extend([
            "--scca",
            params.get("scca", None)
        ])
    if params.get("svd", None) is not None:
        cargs.extend([
            "--svd",
            params.get("svd", None)
        ])
    return cargs


def sccan_outputs(
    params: SccanParamsDict,
    execution: Execution,
) -> SccanOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SccanOutputs(
        root=execution.output_file("."),
    )
    return ret


def sccan_execute(
    params: SccanParamsDict,
    runner: Runner | None = None,
) -> SccanOutputs:
    """
    sccan
    
    A tool for sparse statistical analysis on images : scca, pscca (with
    options), mscca. Can also convert an imagelist/mask pair to a binary matrix
    image.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SccanOutputs`).
    """
    sccan_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SCCAN_METADATA)
    params = execution.params(params)
    cargs = sccan_cargs(params, execution)
    ret = sccan_outputs(params, execution)
    execution.run(cargs)
    return ret


def sccan(
    output: str | None = None,
    n_permutations: int | None = None,
    smoother: int | None = None,
    row_sparseness: int | None = None,
    iterations: int | None = None,
    n_eigenvectors: int | None = None,
    robustify: int | None = None,
    covering: int | None = None,
    uselong: int | None = None,
    l1: float | None = None,
    pclusterthresh: float | None = None,
    qclusterthresh: float | None = None,
    ridge_cca: float | None = None,
    initialization: str | None = None,
    initialization2: str | None = None,
    mask: InputPathType | None = None,
    mask2: InputPathType | None = None,
    partial_scca_option: str | None = None,
    prior_weight: float | None = None,
    get_small: float | None = None,
    verbose: float | None = None,
    imageset_to_matrix: str | None = None,
    timeseriesimage_to_matrix: str | None = None,
    vector_to_image: str | None = None,
    imageset_to_projections: str | None = None,
    scca: str | None = None,
    svd: str | None = None,
    runner: Runner | None = None,
) -> SccanOutputs:
    """
    sccan
    
    A tool for sparse statistical analysis on images : scca, pscca (with
    options), mscca. Can also convert an imagelist/mask pair to a binary matrix
    image.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        output: Output dependent on which option is called.
        n_permutations: Number of permutations to use in scca.
        smoother: Smoothing function for variates.
        row_sparseness: Row sparseness - if (+) then keep values (+) otherwise\
            allow +/- values --- always L1.
        iterations: Max iterations for scca optimization (min 20).
        n_eigenvectors: Number of eigenvectors to compute in scca/spca.
        robustify: Rank-based scca.
        covering: Try to make the decomposition cover the whole domain, if\
            possible.
        uselong: Use longitudinal formulation (> 0) or not (<= 0).
        l1: Use l1 (> 0) or l0 (< 0) penalty, also sets gradient step size.
        pclusterthresh: Cluster threshold on view P.
        qclusterthresh: Cluster threshold on view Q.
        ridge_cca: Ridge cca.
        initialization: Initialization file list for Eigenanatomy - must also\
            pass mask option.
        initialization2: Initialization file list for SCCAN-Eigenanatomy - must\
            also pass mask option.
        mask: Mask file for Eigenanatomy initialization.
        mask2: Mask file for Eigenanatomy initialization 2.
        partial_scca_option: Choices for pscca: PQ, PminusRQ, PQminusR,\
            PminusRQminusR.
        prior_weight: Scalar value weight on prior between 0 (prior is weak)\
            and 1 (prior is strong). Only engaged if initialization is used.
        get_small: Find smallest eigenvectors.
        verbose: Set whether output is verbose.
        imageset_to_matrix: Takes a list of image files names (one per line)\
            and converts it to a 2D matrix/image in binary or csv format.
        timeseriesimage_to_matrix: Takes a timeseries (4D) image and converts\
            it to a 2D matrix csv format.
        vector_to_image: Converts the 1st column vector in a csv file back to\
            an image.
        imageset_to_projections: Takes a list of image and projection files\
            names and writes them to a csv file.
        scca: Matrix-based scca operations for 2 and 3 views.
        svd: A sparse SVD implementation.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SccanOutputs`).
    """
    params = sccan_params(
        output=output,
        n_permutations=n_permutations,
        smoother=smoother,
        row_sparseness=row_sparseness,
        iterations=iterations,
        n_eigenvectors=n_eigenvectors,
        robustify=robustify,
        covering=covering,
        uselong=uselong,
        l1=l1,
        pclusterthresh=pclusterthresh,
        qclusterthresh=qclusterthresh,
        ridge_cca=ridge_cca,
        initialization=initialization,
        initialization2=initialization2,
        mask=mask,
        mask2=mask2,
        partial_scca_option=partial_scca_option,
        prior_weight=prior_weight,
        get_small=get_small,
        verbose=verbose,
        imageset_to_matrix=imageset_to_matrix,
        timeseriesimage_to_matrix=timeseriesimage_to_matrix,
        vector_to_image=vector_to_image,
        imageset_to_projections=imageset_to_projections,
        scca=scca,
        svd=svd,
    )
    return sccan_execute(params, runner)


__all__ = [
    "SCCAN_METADATA",
    "SccanOutputs",
    "SccanParamsDict",
    "SccanParamsDictTagged",
    "sccan",
    "sccan_execute",
    "sccan_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

N3_BIAS_FIELD_CORRECTION_METADATA = Metadata(
    id="9d12f968603c88299a9568f0389d257d580c4fb5.boutiques",
    name="N3BiasFieldCorrection",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_N3BiasFieldCorrectionParamsDictNoTag = typing.TypedDict('_N3BiasFieldCorrectionParamsDictNoTag', {
    "image_dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "input_image": InputPathType,
    "mask_image": typing.NotRequired[InputPathType | None],
    "rescale_intensities": typing.NotRequired[bool | None],
    "weight_image": typing.NotRequired[InputPathType | None],
    "shrink_factor": typing.NotRequired[int | None],
    "convergence": typing.NotRequired[str | None],
    "bspline_fitting": typing.NotRequired[str | None],
    "histogram_sharpening": typing.NotRequired[str | None],
    "output": str,
    "verbose": typing.NotRequired[bool | None],
})
N3BiasFieldCorrectionParamsDictTagged = typing.TypedDict('N3BiasFieldCorrectionParamsDictTagged', {
    "@type": typing.Literal["ants/N3BiasFieldCorrection"],
    "image_dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "input_image": InputPathType,
    "mask_image": typing.NotRequired[InputPathType | None],
    "rescale_intensities": typing.NotRequired[bool | None],
    "weight_image": typing.NotRequired[InputPathType | None],
    "shrink_factor": typing.NotRequired[int | None],
    "convergence": typing.NotRequired[str | None],
    "bspline_fitting": typing.NotRequired[str | None],
    "histogram_sharpening": typing.NotRequired[str | None],
    "output": str,
    "verbose": typing.NotRequired[bool | None],
})
N3BiasFieldCorrectionParamsDict = _N3BiasFieldCorrectionParamsDictNoTag | N3BiasFieldCorrectionParamsDictTagged


class N3BiasFieldCorrectionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `N3BiasFieldCorrectionParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    corrected_image: OutputPathType
    """The bias-corrected version of the input image."""
    bias_field: OutputPathType
    """The estimated bias field, if specified in the output."""


def n3_bias_field_correction_params(
    input_image: InputPathType,
    output: str,
    image_dimensionality: typing.Literal[2, 3, 4] | None = None,
    mask_image: InputPathType | None = None,
    rescale_intensities: bool | None = None,
    weight_image: InputPathType | None = None,
    shrink_factor: int | None = None,
    convergence: str | None = None,
    bspline_fitting: str | None = None,
    histogram_sharpening: str | None = None,
    verbose: bool | None = None,
) -> N3BiasFieldCorrectionParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_image: A scalar image is expected as input for bias correction.\
            Since N3 log transforms the intensities, negative values or values\
            close to zero should be processed prior to correction.
        output: The bias-corrected version of the input image and optionally\
            the estimated bias field.
        image_dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, N3 tries to infer the\
            dimensionality from the input image.
        mask_image: If a mask image is specified, the final bias correction is\
            only performed in the mask region. If a mask image is not specified,\
            the entire image region will be used as the mask region. Note: this\
            differs from the original N3 implementation.
        rescale_intensities: This option rescales the intensity range within\
            the user-specified mask to the original [min, max] range.
        weight_image: The weight image allows the user to perform a relative\
            weighting of specific voxels during the B-spline fitting.
        shrink_factor: Shrink factor to resample the input image. Commonly used\
            values are <= 4.
        convergence: Describes the convergence criteria with default value as\
            [50,0.0].
        bspline_fitting: Describes the parameters for B-Spline fitting.\
            Defaults are [splineDistance,4,3].
        histogram_sharpening: Describes histogram sharpening parameters;\
            defaults are [0.15,0.01,200].
        verbose: Verbose output.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/N3BiasFieldCorrection",
        "input_image": input_image,
        "output": output,
    }
    if image_dimensionality is not None:
        params["image_dimensionality"] = image_dimensionality
    if mask_image is not None:
        params["mask_image"] = mask_image
    if rescale_intensities is not None:
        params["rescale_intensities"] = rescale_intensities
    if weight_image is not None:
        params["weight_image"] = weight_image
    if shrink_factor is not None:
        params["shrink_factor"] = shrink_factor
    if convergence is not None:
        params["convergence"] = convergence
    if bspline_fitting is not None:
        params["bspline_fitting"] = bspline_fitting
    if histogram_sharpening is not None:
        params["histogram_sharpening"] = histogram_sharpening
    if verbose is not None:
        params["verbose"] = verbose
    return params


def n3_bias_field_correction_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `N3BiasFieldCorrectionParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("image_dimensionality", None) is not None:
        if not isinstance(params["image_dimensionality"], int):
            raise StyxValidationError(f'`image_dimensionality` has the wrong type: Received `{type(params.get("image_dimensionality", None))}` expected `typing.Literal[2, 3, 4] | None`')
        if params["image_dimensionality"] not in [2, 3, 4]:
            raise StyxValidationError("Parameter `image_dimensionality` must be one of [2, 3, 4]")
    if params.get("input_image", None) is None:
        raise StyxValidationError("`input_image` must not be None")
    if not isinstance(params["input_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_image` has the wrong type: Received `{type(params.get("input_image", None))}` expected `InputPathType`')
    if params.get("mask_image", None) is not None:
        if not isinstance(params["mask_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask_image` has the wrong type: Received `{type(params.get("mask_image", None))}` expected `InputPathType | None`')
    if params.get("rescale_intensities", None) is not None:
        if not isinstance(params["rescale_intensities"], bool):
            raise StyxValidationError(f'`rescale_intensities` has the wrong type: Received `{type(params.get("rescale_intensities", None))}` expected `bool | None`')
    if params.get("weight_image", None) is not None:
        if not isinstance(params["weight_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`weight_image` has the wrong type: Received `{type(params.get("weight_image", None))}` expected `InputPathType | None`')
    if params.get("shrink_factor", None) is not None:
        if not isinstance(params["shrink_factor"], int):
            raise StyxValidationError(f'`shrink_factor` has the wrong type: Received `{type(params.get("shrink_factor", None))}` expected `int | None`')
    if params.get("convergence", None) is not None:
        if not isinstance(params["convergence"], str):
            raise StyxValidationError(f'`convergence` has the wrong type: Received `{type(params.get("convergence", None))}` expected `str | None`')
    if params.get("bspline_fitting", None) is not None:
        if not isinstance(params["bspline_fitting"], str):
            raise StyxValidationError(f'`bspline_fitting` has the wrong type: Received `{type(params.get("bspline_fitting", None))}` expected `str | None`')
    if params.get("histogram_sharpening", None) is not None:
        if not isinstance(params["histogram_sharpening"], str):
            raise StyxValidationError(f'`histogram_sharpening` has the wrong type: Received `{type(params.get("histogram_sharpening", None))}` expected `str | None`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')
    if params.get("verbose", None) is not None:
        if not isinstance(params["verbose"], bool):
            raise StyxValidationError(f'`verbose` has the wrong type: Received `{type(params.get("verbose", None))}` expected `bool | None`')


def n3_bias_field_correction_cargs(
    params: N3BiasFieldCorrectionParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("N3BiasFieldCorrection")
    if params.get("image_dimensionality", None) is not None:
        cargs.extend([
            "--image-dimensionality",
            str(params.get("image_dimensionality", None))
        ])
    cargs.extend([
        "--input-image",
        execution.input_file(params.get("input_image", None))
    ])
    if params.get("mask_image", None) is not None:
        cargs.extend([
            "--mask-image",
            execution.input_file(params.get("mask_image", None))
        ])
    if params.get("rescale_intensities", None) is not None:
        cargs.extend([
            "--rescale-intensities",
            ("1" if params.get("rescale_intensities", None) else "0")
        ])
    if params.get("weight_image", None) is not None:
        cargs.extend([
            "--weight-image",
            execution.input_file(params.get("weight_image", None))
        ])
    if params.get("shrink_factor", None) is not None:
        cargs.extend([
            "--shrink-factor",
            str(params.get("shrink_factor", None))
        ])
    if params.get("convergence", None) is not None:
        cargs.extend([
            "--convergence",
            params.get("convergence", None)
        ])
    if params.get("bspline_fitting", None) is not None:
        cargs.extend([
            "--bspline-fitting",
            params.get("bspline_fitting", None)
        ])
    if params.get("histogram_sharpening", None) is not None:
        cargs.extend([
            "--histogram-sharpening",
            params.get("histogram_sharpening", None)
        ])
    cargs.extend([
        "--output",
        params.get("output", None)
    ])
    if params.get("verbose", None) is not None:
        cargs.extend([
            "--verbose",
            ("1" if params.get("verbose", None) else "0")
        ])
    return cargs


def n3_bias_field_correction_outputs(
    params: N3BiasFieldCorrectionParamsDict,
    execution: Execution,
) -> N3BiasFieldCorrectionOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = N3BiasFieldCorrectionOutputs(
        root=execution.output_file("."),
        corrected_image=execution.output_file(params.get("output", None)),
        bias_field=execution.output_file("[BIS_FIELD]"),
    )
    return ret


def n3_bias_field_correction_execute(
    params: N3BiasFieldCorrectionParamsDict,
    runner: Runner | None = None,
) -> N3BiasFieldCorrectionOutputs:
    """
    N3BiasFieldCorrection
    
    This N3 is a variant of the popular N3 (nonparametric nonuniform
    normalization) retrospective bias correction algorithm. Based on the
    assumption that the corruption of the low frequency bias field can be
    modeled as a convolution of the intensity histogram by a Gaussian, the basic
    algorithmic protocol is to iterate between deconvolving the intensity
    histogram by a Gaussian, remapping the intensities, and then spatially
    smoothing this result by a B-spline modeling of the bias field itself.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `N3BiasFieldCorrectionOutputs`).
    """
    n3_bias_field_correction_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(N3_BIAS_FIELD_CORRECTION_METADATA)
    params = execution.params(params)
    cargs = n3_bias_field_correction_cargs(params, execution)
    ret = n3_bias_field_correction_outputs(params, execution)
    execution.run(cargs)
    return ret


def n3_bias_field_correction(
    input_image: InputPathType,
    output: str,
    image_dimensionality: typing.Literal[2, 3, 4] | None = None,
    mask_image: InputPathType | None = None,
    rescale_intensities: bool | None = None,
    weight_image: InputPathType | None = None,
    shrink_factor: int | None = None,
    convergence: str | None = None,
    bspline_fitting: str | None = None,
    histogram_sharpening: str | None = None,
    verbose: bool | None = None,
    runner: Runner | None = None,
) -> N3BiasFieldCorrectionOutputs:
    """
    N3BiasFieldCorrection
    
    This N3 is a variant of the popular N3 (nonparametric nonuniform
    normalization) retrospective bias correction algorithm. Based on the
    assumption that the corruption of the low frequency bias field can be
    modeled as a convolution of the intensity histogram by a Gaussian, the basic
    algorithmic protocol is to iterate between deconvolving the intensity
    histogram by a Gaussian, remapping the intensities, and then spatially
    smoothing this result by a B-spline modeling of the bias field itself.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        input_image: A scalar image is expected as input for bias correction.\
            Since N3 log transforms the intensities, negative values or values\
            close to zero should be processed prior to correction.
        output: The bias-corrected version of the input image and optionally\
            the estimated bias field.
        image_dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, N3 tries to infer the\
            dimensionality from the input image.
        mask_image: If a mask image is specified, the final bias correction is\
            only performed in the mask region. If a mask image is not specified,\
            the entire image region will be used as the mask region. Note: this\
            differs from the original N3 implementation.
        rescale_intensities: This option rescales the intensity range within\
            the user-specified mask to the original [min, max] range.
        weight_image: The weight image allows the user to perform a relative\
            weighting of specific voxels during the B-spline fitting.
        shrink_factor: Shrink factor to resample the input image. Commonly used\
            values are <= 4.
        convergence: Describes the convergence criteria with default value as\
            [50,0.0].
        bspline_fitting: Describes the parameters for B-Spline fitting.\
            Defaults are [splineDistance,4,3].
        histogram_sharpening: Describes histogram sharpening parameters;\
            defaults are [0.15,0.01,200].
        verbose: Verbose output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `N3BiasFieldCorrectionOutputs`).
    """
    params = n3_bias_field_correction_params(
        image_dimensionality=image_dimensionality,
        input_image=input_image,
        mask_image=mask_image,
        rescale_intensities=rescale_intensities,
        weight_image=weight_image,
        shrink_factor=shrink_factor,
        convergence=convergence,
        bspline_fitting=bspline_fitting,
        histogram_sharpening=histogram_sharpening,
        output=output,
        verbose=verbose,
    )
    return n3_bias_field_correction_execute(params, runner)


__all__ = [
    "N3BiasFieldCorrectionOutputs",
    "N3BiasFieldCorrectionParamsDict",
    "N3BiasFieldCorrectionParamsDictTagged",
    "N3_BIAS_FIELD_CORRECTION_METADATA",
    "n3_bias_field_correction",
    "n3_bias_field_correction_execute",
    "n3_bias_field_correction_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CONVERT_TO_JPG_METADATA = Metadata(
    id="05ba9d9a81340f45633926fc4c293fdf619a95ab.boutiques",
    name="ConvertToJpg",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_ConvertToJpgParamsDictNoTag = typing.TypedDict('_ConvertToJpgParamsDictNoTag', {
    "infile": InputPathType,
    "outfile": str,
})
ConvertToJpgParamsDictTagged = typing.TypedDict('ConvertToJpgParamsDictTagged', {
    "@type": typing.Literal["ants/ConvertToJpg"],
    "infile": InputPathType,
    "outfile": str,
})
ConvertToJpgParamsDict = _ConvertToJpgParamsDictNoTag | ConvertToJpgParamsDictTagged


class ConvertToJpgOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ConvertToJpgParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_jpg: OutputPathType
    """The converted JPG image."""


def convert_to_jpg_params(
    infile: InputPathType,
    outfile: str,
) -> ConvertToJpgParamsDictTagged:
    """
    Build parameters.
    
    Args:
        infile: The input file in NIfTI format.
        outfile: The output file in JPG format.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/ConvertToJpg",
        "infile": infile,
        "outfile": outfile,
    }
    return params


def convert_to_jpg_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertToJpgParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("infile", None) is None:
        raise StyxValidationError("`infile` must not be None")
    if not isinstance(params["infile"], (pathlib.Path, str)):
        raise StyxValidationError(f'`infile` has the wrong type: Received `{type(params.get("infile", None))}` expected `InputPathType`')
    if params.get("outfile", None) is None:
        raise StyxValidationError("`outfile` must not be None")
    if not isinstance(params["outfile"], str):
        raise StyxValidationError(f'`outfile` has the wrong type: Received `{type(params.get("outfile", None))}` expected `str`')


def convert_to_jpg_cargs(
    params: ConvertToJpgParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ConvertToJpg")
    cargs.append(execution.input_file(params.get("infile", None)))
    cargs.append(params.get("outfile", None))
    return cargs


def convert_to_jpg_outputs(
    params: ConvertToJpgParamsDict,
    execution: Execution,
) -> ConvertToJpgOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ConvertToJpgOutputs(
        root=execution.output_file("."),
        output_jpg=execution.output_file(params.get("outfile", None)),
    )
    return ret


def convert_to_jpg_execute(
    params: ConvertToJpgParamsDict,
    runner: Runner | None = None,
) -> ConvertToJpgOutputs:
    """
    ConvertToJpg
    
    A tool to convert NIfTI images to JPG format.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertToJpgOutputs`).
    """
    convert_to_jpg_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONVERT_TO_JPG_METADATA)
    params = execution.params(params)
    cargs = convert_to_jpg_cargs(params, execution)
    ret = convert_to_jpg_outputs(params, execution)
    execution.run(cargs)
    return ret


def convert_to_jpg(
    infile: InputPathType,
    outfile: str,
    runner: Runner | None = None,
) -> ConvertToJpgOutputs:
    """
    ConvertToJpg
    
    A tool to convert NIfTI images to JPG format.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        infile: The input file in NIfTI format.
        outfile: The output file in JPG format.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertToJpgOutputs`).
    """
    params = convert_to_jpg_params(
        infile=infile,
        outfile=outfile,
    )
    return convert_to_jpg_execute(params, runner)


__all__ = [
    "CONVERT_TO_JPG_METADATA",
    "ConvertToJpgOutputs",
    "ConvertToJpgParamsDict",
    "ConvertToJpgParamsDictTagged",
    "convert_to_jpg",
    "convert_to_jpg_execute",
    "convert_to_jpg_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_JOINT_TENSOR_FUSION_METADATA = Metadata(
    id="5d343b97b1984cf23521db0f60519ddbabba3a2e.boutiques",
    name="antsJointTensorFusion",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


_AntsJointTensorFusionParamsDictNoTag = typing.TypedDict('_AntsJointTensorFusionParamsDictNoTag', {
    "dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "target_image": list[str],
    "atlas_image": list[str],
    "atlas_segmentation": InputPathType,
    "alpha": typing.NotRequired[float | None],
    "beta": typing.NotRequired[float | None],
    "retain_label_posterior_images": typing.NotRequired[bool | None],
    "retain_atlas_voting_images": typing.NotRequired[bool | None],
    "constrain_nonnegative": typing.NotRequired[bool | None],
    "log_euclidean": typing.NotRequired[bool | None],
    "patch_radius": typing.NotRequired[str | None],
    "patch_metric": typing.NotRequired[typing.Literal["PC", "MSQ"] | None],
    "search_radius": typing.NotRequired[str | None],
    "exclusion_image": typing.NotRequired[str | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "output": str,
    "verbose": typing.NotRequired[bool | None],
})
AntsJointTensorFusionParamsDictTagged = typing.TypedDict('AntsJointTensorFusionParamsDictTagged', {
    "@type": typing.Literal["ants/antsJointTensorFusion"],
    "dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "target_image": list[str],
    "atlas_image": list[str],
    "atlas_segmentation": InputPathType,
    "alpha": typing.NotRequired[float | None],
    "beta": typing.NotRequired[float | None],
    "retain_label_posterior_images": typing.NotRequired[bool | None],
    "retain_atlas_voting_images": typing.NotRequired[bool | None],
    "constrain_nonnegative": typing.NotRequired[bool | None],
    "log_euclidean": typing.NotRequired[bool | None],
    "patch_radius": typing.NotRequired[str | None],
    "patch_metric": typing.NotRequired[typing.Literal["PC", "MSQ"] | None],
    "search_radius": typing.NotRequired[str | None],
    "exclusion_image": typing.NotRequired[str | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "output": str,
    "verbose": typing.NotRequired[bool | None],
})
AntsJointTensorFusionParamsDict = _AntsJointTensorFusionParamsDictNoTag | AntsJointTensorFusionParamsDictTagged


class AntsJointTensorFusionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsJointTensorFusionParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    label_fusion_image: OutputPathType
    """The label fusion image output."""
    intensity_fusion_image: OutputPathType
    """The intensity fusion image output."""
    label_posterior_probability_image: OutputPathType
    """The label posterior probability images."""
    atlas_voting_weight_image: OutputPathType
    """The atlas voting weight images."""


def ants_joint_tensor_fusion_params(
    target_image: list[str],
    atlas_image: list[str],
    atlas_segmentation: InputPathType,
    output: str,
    dimensionality: typing.Literal[2, 3, 4] | None = None,
    alpha: float | None = None,
    beta: float | None = None,
    retain_label_posterior_images: bool | None = None,
    retain_atlas_voting_images: bool | None = None,
    constrain_nonnegative: bool | None = None,
    log_euclidean: bool | None = None,
    patch_radius: str | None = None,
    patch_metric: typing.Literal["PC", "MSQ"] | None = None,
    search_radius: str | None = None,
    exclusion_image: str | None = None,
    mask_image: InputPathType | None = None,
    verbose: bool | None = None,
) -> AntsJointTensorFusionParamsDictTagged:
    """
    Build parameters.
    
    Args:
        target_image: The target image (or multimodal target images) assumed to\
            be aligned to a common image domain.
        atlas_image: The atlas image (or multimodal atlas images) assumed to be\
            aligned to a common image domain.
        atlas_segmentation: The atlas segmentation images. For performing label\
            fusion the number of specified segmentations should be identical to the\
            number of atlas image sets.
        output: The output is the intensity and/or label fusion image.\
            Additional optional outputs include the label posterior probability\
            images and the atlas voting weight images.
        dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, the program tries to\
            infer the dimensionality from the input image.
        alpha: Regularization term added to matrix Mx for calculating the\
            inverse. Default = 0.1.
        beta: Exponent for mapping intensity difference to the joint error.\
            Default = 2.0.
        retain_label_posterior_images: Retain label posterior probability\
            images. Requires atlas segmentations to be specified. Default = false.
        retain_atlas_voting_images: Retain atlas voting images. Default = false.
        constrain_nonnegative: Constrain solution to non-negative weights.
        log_euclidean: Use log Euclidean space for tensor math.
        patch_radius: Patch radius for similarity measures. Default = 2x2x2.
        patch_metric: Metric to be used in determining the most similar\
            neighborhood patch. Options include Pearson's correlation (PC) and mean\
            squares (MSQ). Default = PC.
        search_radius: Search radius for similarity measures. Default = 3x3x3.
        exclusion_image: Specify an exclusion region for the given label.
        mask_image: If a mask image is specified, fusion is only performed in\
            the mask region.
        verbose: Verbose output.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ants/antsJointTensorFusion",
        "target_image": target_image,
        "atlas_image": atlas_image,
        "atlas_segmentation": atlas_segmentation,
        "output": output,
    }
    if dimensionality is not None:
        params["dimensionality"] = dimensionality
    if alpha is not None:
        params["alpha"] = alpha
    if beta is not None:
        params["beta"] = beta
    if retain_label_posterior_images is not None:
        params["retain_label_posterior_images"] = retain_label_posterior_images
    if retain_atlas_voting_images is not None:
        params["retain_atlas_voting_images"] = retain_atlas_voting_images
    if constrain_nonnegative is not None:
        params["constrain_nonnegative"] = constrain_nonnegative
    if log_euclidean is not None:
        params["log_euclidean"] = log_euclidean
    if patch_radius is not None:
        params["patch_radius"] = patch_radius
    if patch_metric is not None:
        params["patch_metric"] = patch_metric
    if search_radius is not None:
        params["search_radius"] = search_radius
    if exclusion_image is not None:
        params["exclusion_image"] = exclusion_image
    if mask_image is not None:
        params["mask_image"] = mask_image
    if verbose is not None:
        params["verbose"] = verbose
    return params


def ants_joint_tensor_fusion_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AntsJointTensorFusionParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("dimensionality", None) is not None:
        if not isinstance(params["dimensionality"], int):
            raise StyxValidationError(f'`dimensionality` has the wrong type: Received `{type(params.get("dimensionality", None))}` expected `typing.Literal[2, 3, 4] | None`')
        if params["dimensionality"] not in [2, 3, 4]:
            raise StyxValidationError("Parameter `dimensionality` must be one of [2, 3, 4]")
    if params.get("target_image", None) is None:
        raise StyxValidationError("`target_image` must not be None")
    if not isinstance(params["target_image"], list):
        raise StyxValidationError(f'`target_image` has the wrong type: Received `{type(params.get("target_image", None))}` expected `list[str]`')
    for e in params["target_image"]:
        if not isinstance(e, str):
            raise StyxValidationError(f'`target_image` has the wrong type: Received `{type(params.get("target_image", None))}` expected `list[str]`')
    if params.get("atlas_image", None) is None:
        raise StyxValidationError("`atlas_image` must not be None")
    if not isinstance(params["atlas_image"], list):
        raise StyxValidationError(f'`atlas_image` has the wrong type: Received `{type(params.get("atlas_image", None))}` expected `list[str]`')
    for e in params["atlas_image"]:
        if not isinstance(e, str):
            raise StyxValidationError(f'`atlas_image` has the wrong type: Received `{type(params.get("atlas_image", None))}` expected `list[str]`')
    if params.get("atlas_segmentation", None) is None:
        raise StyxValidationError("`atlas_segmentation` must not be None")
    if not isinstance(params["atlas_segmentation"], (pathlib.Path, str)):
        raise StyxValidationError(f'`atlas_segmentation` has the wrong type: Received `{type(params.get("atlas_segmentation", None))}` expected `InputPathType`')
    if params.get("alpha", None) is not None:
        if not isinstance(params["alpha"], (float, int)):
            raise StyxValidationError(f'`alpha` has the wrong type: Received `{type(params.get("alpha", None))}` expected `float | None`')
    if params.get("beta", None) is not None:
        if not isinstance(params["beta"], (float, int)):
            raise StyxValidationError(f'`beta` has the wrong type: Received `{type(params.get("beta", None))}` expected `float | None`')
    if params.get("retain_label_posterior_images", None) is not None:
        if not isinstance(params["retain_label_posterior_images"], bool):
            raise StyxValidationError(f'`retain_label_posterior_images` has the wrong type: Received `{type(params.get("retain_label_posterior_images", None))}` expected `bool | None`')
    if params.get("retain_atlas_voting_images", None) is not None:
        if not isinstance(params["retain_atlas_voting_images"], bool):
            raise StyxValidationError(f'`retain_atlas_voting_images` has the wrong type: Received `{type(params.get("retain_atlas_voting_images", None))}` expected `bool | None`')
    if params.get("constrain_nonnegative", None) is not None:
        if not isinstance(params["constrain_nonnegative"], bool):
            raise StyxValidationError(f'`constrain_nonnegative` has the wrong type: Received `{type(params.get("constrain_nonnegative", None))}` expected `bool | None`')
    if params.get("log_euclidean", None) is not None:
        if not isinstance(params["log_euclidean"], bool):
            raise StyxValidationError(f'`log_euclidean` has the wrong type: Received `{type(params.get("log_euclidean", None))}` expected `bool | None`')
    if params.get("patch_radius", None) is not None:
        if not isinstance(params["patch_radius"], str):
            raise StyxValidationError(f'`patch_radius` has the wrong type: Received `{type(params.get("patch_radius", None))}` expected `str | None`')
    if params.get("patch_metric", None) is not None:
        if not isinstance(params["patch_metric"], str):
            raise StyxValidationError(f'`patch_metric` has the wrong type: Received `{type(params.get("patch_metric", None))}` expected `typing.Literal["PC", "MSQ"] | None`')
        if params["patch_metric"] not in ["PC", "MSQ"]:
            raise StyxValidationError("Parameter `patch_metric` must be one of [\"PC\", \"MSQ\"]")
    if params.get("search_radius", None) is not None:
        if not isinstance(params["search_radius"], str):
            raise StyxValidationError(f'`search_radius` has the wrong type: Received `{type(params.get("search_radius", None))}` expected `str | None`')
    if params.get("exclusion_image", None) is not None:
        if not isinstance(params["exclusion_image"], str):
            raise StyxValidationError(f'`exclusion_image` has the wrong type: Received `{type(params.get("exclusion_image", None))}` expected `str | None`')
    if params.get("mask_image", None) is not None:
        if not isinstance(params["mask_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask_image` has the wrong type: Received `{type(params.get("mask_image", None))}` expected `InputPathType | None`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')
    if params.get("verbose", None) is not None:
        if not isinstance(params["verbose"], bool):
            raise StyxValidationError(f'`verbose` has the wrong type: Received `{type(params.get("verbose", None))}` expected `bool | None`')


def ants_joint_tensor_fusion_cargs(
    params: AntsJointTensorFusionParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsJointTensorFusion")
    if params.get("dimensionality", None) is not None:
        cargs.extend([
            "--image-dimensionality",
            str(params.get("dimensionality", None))
        ])
    cargs.extend([
        "-t",
        ",".join(params.get("target_image", None))
    ])
    cargs.extend([
        "-g",
        ",".join(params.get("atlas_image", None))
    ])
    cargs.extend([
        "-l",
        execution.input_file(params.get("atlas_segmentation", None))
    ])
    if params.get("alpha", None) is not None:
        cargs.extend([
            "-a",
            str(params.get("alpha", None))
        ])
    if params.get("beta", None) is not None:
        cargs.extend([
            "-b",
            str(params.get("beta", None))
        ])
    if params.get("retain_label_posterior_images", None) is not None:
        cargs.extend([
            "-r",
            ("1" if params.get("retain_label_posterior_images", None) else "0")
        ])
    if params.get("retain_atlas_voting_images", None) is not None:
        cargs.extend([
            "-f",
            ("1" if params.get("retain_atlas_voting_images", None) else "0")
        ])
    if params.get("constrain_nonnegative", None) is not None:
        cargs.extend([
            "-c",
            ("1" if params.get("constrain_nonnegative", None) else "0")
        ])
    if params.get("log_euclidean", None) is not None:
        cargs.extend([
            "-u",
            ("1" if params.get("log_euclidean", None) else "0")
        ])
    if params.get("patch_radius", None) is not None:
        cargs.extend([
            "-p",
            params.get("patch_radius", None)
        ])
    if params.get("patch_metric", None) is not None:
        cargs.extend([
            "-m",
            params.get("patch_metric", None)
        ])
    if params.get("search_radius", None) is not None:
        cargs.extend([
            "-s",
            params.get("search_radius", None)
        ])
    if params.get("exclusion_image", None) is not None:
        cargs.extend([
            "-e",
            params.get("exclusion_image", None)
        ])
    if params.get("mask_image", None) is not None:
        cargs.extend([
            "-x",
            execution.input_file(params.get("mask_image", None))
        ])
    cargs.extend([
        "-o",
        params.get("output", None)
    ])
    if params.get("verbose", None) is not None:
        cargs.extend([
            "-v",
            ("1" if params.get("verbose", None) else "0")
        ])
    return cargs


def ants_joint_tensor_fusion_outputs(
    params: AntsJointTensorFusionParamsDict,
    execution: Execution,
) -> AntsJointTensorFusionOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsJointTensorFusionOutputs(
        root=execution.output_file("."),
        label_fusion_image=execution.output_file(params.get("output", None) + "_LabelFusion.nii.gz"),
        intensity_fusion_image=execution.output_file(params.get("output", None) + "_IntensityFusion.nii.gz"),
        label_posterior_probability_image=execution.output_file(params.get("output", None) + "_LabelPosterior.nii.gz"),
        atlas_voting_weight_image=execution.output_file(params.get("output", None) + "_AtlasVoting.nii.gz"),
    )
    return ret


def ants_joint_tensor_fusion_execute(
    params: AntsJointTensorFusionParamsDict,
    runner: Runner | None = None,
) -> AntsJointTensorFusionOutputs:
    """
    antsJointTensorFusion
    
    antsJointTensorFusion is an image fusion algorithm developed by Hongzhi Wang
    and Paul Yushkevich which won segmentation challenges at MICCAI 2012 and
    MICCAI 2013. The original label fusion framework was extended to accommodate
    intensities by Brian Avants. This implementation is based on the original
    ITK-style implementation and ANTsR implementation.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsJointTensorFusionOutputs`).
    """
    ants_joint_tensor_fusion_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_JOINT_TENSOR_FUSION_METADATA)
    params = execution.params(params)
    cargs = ants_joint_tensor_fusion_cargs(params, execution)
    ret = ants_joint_tensor_fusion_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_joint_tensor_fusion(
    target_image: list[str],
    atlas_image: list[str],
    atlas_segmentation: InputPathType,
    output: str,
    dimensionality: typing.Literal[2, 3, 4] | None = None,
    alpha: float | None = None,
    beta: float | None = None,
    retain_label_posterior_images: bool | None = None,
    retain_atlas_voting_images: bool | None = None,
    constrain_nonnegative: bool | None = None,
    log_euclidean: bool | None = None,
    patch_radius: str | None = None,
    patch_metric: typing.Literal["PC", "MSQ"] | None = None,
    search_radius: str | None = None,
    exclusion_image: str | None = None,
    mask_image: InputPathType | None = None,
    verbose: bool | None = None,
    runner: Runner | None = None,
) -> AntsJointTensorFusionOutputs:
    """
    antsJointTensorFusion
    
    antsJointTensorFusion is an image fusion algorithm developed by Hongzhi Wang
    and Paul Yushkevich which won segmentation challenges at MICCAI 2012 and
    MICCAI 2013. The original label fusion framework was extended to accommodate
    intensities by Brian Avants. This implementation is based on the original
    ITK-style implementation and ANTsR implementation.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        target_image: The target image (or multimodal target images) assumed to\
            be aligned to a common image domain.
        atlas_image: The atlas image (or multimodal atlas images) assumed to be\
            aligned to a common image domain.
        atlas_segmentation: The atlas segmentation images. For performing label\
            fusion the number of specified segmentations should be identical to the\
            number of atlas image sets.
        output: The output is the intensity and/or label fusion image.\
            Additional optional outputs include the label posterior probability\
            images and the atlas voting weight images.
        dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, the program tries to\
            infer the dimensionality from the input image.
        alpha: Regularization term added to matrix Mx for calculating the\
            inverse. Default = 0.1.
        beta: Exponent for mapping intensity difference to the joint error.\
            Default = 2.0.
        retain_label_posterior_images: Retain label posterior probability\
            images. Requires atlas segmentations to be specified. Default = false.
        retain_atlas_voting_images: Retain atlas voting images. Default = false.
        constrain_nonnegative: Constrain solution to non-negative weights.
        log_euclidean: Use log Euclidean space for tensor math.
        patch_radius: Patch radius for similarity measures. Default = 2x2x2.
        patch_metric: Metric to be used in determining the most similar\
            neighborhood patch. Options include Pearson's correlation (PC) and mean\
            squares (MSQ). Default = PC.
        search_radius: Search radius for similarity measures. Default = 3x3x3.
        exclusion_image: Specify an exclusion region for the given label.
        mask_image: If a mask image is specified, fusion is only performed in\
            the mask region.
        verbose: Verbose output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsJointTensorFusionOutputs`).
    """
    params = ants_joint_tensor_fusion_params(
        dimensionality=dimensionality,
        target_image=target_image,
        atlas_image=atlas_image,
        atlas_segmentation=atlas_segmentation,
        alpha=alpha,
        beta=beta,
        retain_label_posterior_images=retain_label_posterior_images,
        retain_atlas_voting_images=retain_atlas_voting_images,
        constrain_nonnegative=constrain_nonnegative,
        log_euclidean=log_euclidean,
        patch_radius=patch_radius,
        patch_metric=patch_metric,
        search_radius=search_radius,
        exclusion_image=exclusion_image,
        mask_image=mask_image,
        output=output,
        verbose=verbose,
    )
    return ants_joint_tensor_fusion_execute(params, runner)


__all__ = [
    "ANTS_JOINT_TENSOR_FUSION_METADATA",
    "AntsJointTensorFusionOutputs",
    "AntsJointTensorFusionParamsDict",
    "AntsJointTensorFusionParamsDictTagged",
    "ants_joint_tensor_fusion",
    "ants_joint_tensor_fusion_execute",
    "ants_joint_tensor_fusion_params",
]
